# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['ClientUserDefineRuleArgs', 'ClientUserDefineRule']

@pulumi.input_type
class ClientUserDefineRuleArgs:
    def __init__(__self__, *,
                 action_type: pulumi.Input[int],
                 client_user_define_rule_name: pulumi.Input[str],
                 platform: pulumi.Input[str],
                 type: pulumi.Input[int],
                 cmdline: Optional[pulumi.Input[str]] = None,
                 file_path: Optional[pulumi.Input[str]] = None,
                 hash: Optional[pulumi.Input[str]] = None,
                 ip: Optional[pulumi.Input[str]] = None,
                 new_file_path: Optional[pulumi.Input[str]] = None,
                 parent_cmdline: Optional[pulumi.Input[str]] = None,
                 parent_proc_path: Optional[pulumi.Input[str]] = None,
                 port_str: Optional[pulumi.Input[str]] = None,
                 proc_path: Optional[pulumi.Input[str]] = None,
                 registry_content: Optional[pulumi.Input[str]] = None,
                 registry_key: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a ClientUserDefineRule resource.
        :param pulumi.Input[int] action_type: The operation type. Value:
               - **0**: plus White
               - **1**: Plus Black.
        :param pulumi.Input[str] client_user_define_rule_name: The custom rule name.
        :param pulumi.Input[str] platform: The operating system type. Value:
               - **windows**:widows
               - **linux**:linux
               - **all**: all.
        :param pulumi.Input[int] type: The rule type. Value:
               - **1**: Process hash
               - **2**: command line
               - **3**: Process network
               - **4**: File reading and writing
               - **5**: Operate the registry
               - **6**: Load Dynamic Link Library
               - **7**: File Rename.
        :param pulumi.Input[str] cmdline: Command line. When the value of the Type attribute is 2, 3, 4, 5, 6, or 7, the command line field is required.
        :param pulumi.Input[str] file_path: The file path. When the value of the Type attribute is 4 or 6, 7, the FilePath field is required.
        :param pulumi.Input[str] hash: Process hash list. When the value of the Type attribute is 1, the Hash attribute is required.
        :param pulumi.Input[str] ip: IP address. When the value of the Type attribute is 3, the Ip attribute is required.
        :param pulumi.Input[str] new_file_path: The new file path to rename the file. When the value of the Type attribute is 7, the NewFilePath attribute is required.
        :param pulumi.Input[str] parent_cmdline: The parent command line.
        :param pulumi.Input[str] parent_proc_path: Parent process path.
        :param pulumi.Input[str] port_str: The port number. When the value of the Type attribute is 3, the PortStr attribute is required. Value range: **1-65535**.
        :param pulumi.Input[str] proc_path: The process path. When the Type attribute is set to 2, 3, 4, 5, 6, or 7, the ProcPath attribute is required.
        :param pulumi.Input[str] registry_content: The registry value. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        :param pulumi.Input[str] registry_key: The registry key. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        """
        pulumi.set(__self__, "action_type", action_type)
        pulumi.set(__self__, "client_user_define_rule_name", client_user_define_rule_name)
        pulumi.set(__self__, "platform", platform)
        pulumi.set(__self__, "type", type)
        if cmdline is not None:
            pulumi.set(__self__, "cmdline", cmdline)
        if file_path is not None:
            pulumi.set(__self__, "file_path", file_path)
        if hash is not None:
            pulumi.set(__self__, "hash", hash)
        if ip is not None:
            pulumi.set(__self__, "ip", ip)
        if new_file_path is not None:
            pulumi.set(__self__, "new_file_path", new_file_path)
        if parent_cmdline is not None:
            pulumi.set(__self__, "parent_cmdline", parent_cmdline)
        if parent_proc_path is not None:
            pulumi.set(__self__, "parent_proc_path", parent_proc_path)
        if port_str is not None:
            pulumi.set(__self__, "port_str", port_str)
        if proc_path is not None:
            pulumi.set(__self__, "proc_path", proc_path)
        if registry_content is not None:
            pulumi.set(__self__, "registry_content", registry_content)
        if registry_key is not None:
            pulumi.set(__self__, "registry_key", registry_key)

    @property
    @pulumi.getter(name="actionType")
    def action_type(self) -> pulumi.Input[int]:
        """
        The operation type. Value:
        - **0**: plus White
        - **1**: Plus Black.
        """
        return pulumi.get(self, "action_type")

    @action_type.setter
    def action_type(self, value: pulumi.Input[int]):
        pulumi.set(self, "action_type", value)

    @property
    @pulumi.getter(name="clientUserDefineRuleName")
    def client_user_define_rule_name(self) -> pulumi.Input[str]:
        """
        The custom rule name.
        """
        return pulumi.get(self, "client_user_define_rule_name")

    @client_user_define_rule_name.setter
    def client_user_define_rule_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "client_user_define_rule_name", value)

    @property
    @pulumi.getter
    def platform(self) -> pulumi.Input[str]:
        """
        The operating system type. Value:
        - **windows**:widows
        - **linux**:linux
        - **all**: all.
        """
        return pulumi.get(self, "platform")

    @platform.setter
    def platform(self, value: pulumi.Input[str]):
        pulumi.set(self, "platform", value)

    @property
    @pulumi.getter
    def type(self) -> pulumi.Input[int]:
        """
        The rule type. Value:
        - **1**: Process hash
        - **2**: command line
        - **3**: Process network
        - **4**: File reading and writing
        - **5**: Operate the registry
        - **6**: Load Dynamic Link Library
        - **7**: File Rename.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: pulumi.Input[int]):
        pulumi.set(self, "type", value)

    @property
    @pulumi.getter
    def cmdline(self) -> Optional[pulumi.Input[str]]:
        """
        Command line. When the value of the Type attribute is 2, 3, 4, 5, 6, or 7, the command line field is required.
        """
        return pulumi.get(self, "cmdline")

    @cmdline.setter
    def cmdline(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "cmdline", value)

    @property
    @pulumi.getter(name="filePath")
    def file_path(self) -> Optional[pulumi.Input[str]]:
        """
        The file path. When the value of the Type attribute is 4 or 6, 7, the FilePath field is required.
        """
        return pulumi.get(self, "file_path")

    @file_path.setter
    def file_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "file_path", value)

    @property
    @pulumi.getter
    def hash(self) -> Optional[pulumi.Input[str]]:
        """
        Process hash list. When the value of the Type attribute is 1, the Hash attribute is required.
        """
        return pulumi.get(self, "hash")

    @hash.setter
    def hash(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "hash", value)

    @property
    @pulumi.getter
    def ip(self) -> Optional[pulumi.Input[str]]:
        """
        IP address. When the value of the Type attribute is 3, the Ip attribute is required.
        """
        return pulumi.get(self, "ip")

    @ip.setter
    def ip(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ip", value)

    @property
    @pulumi.getter(name="newFilePath")
    def new_file_path(self) -> Optional[pulumi.Input[str]]:
        """
        The new file path to rename the file. When the value of the Type attribute is 7, the NewFilePath attribute is required.
        """
        return pulumi.get(self, "new_file_path")

    @new_file_path.setter
    def new_file_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "new_file_path", value)

    @property
    @pulumi.getter(name="parentCmdline")
    def parent_cmdline(self) -> Optional[pulumi.Input[str]]:
        """
        The parent command line.
        """
        return pulumi.get(self, "parent_cmdline")

    @parent_cmdline.setter
    def parent_cmdline(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "parent_cmdline", value)

    @property
    @pulumi.getter(name="parentProcPath")
    def parent_proc_path(self) -> Optional[pulumi.Input[str]]:
        """
        Parent process path.
        """
        return pulumi.get(self, "parent_proc_path")

    @parent_proc_path.setter
    def parent_proc_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "parent_proc_path", value)

    @property
    @pulumi.getter(name="portStr")
    def port_str(self) -> Optional[pulumi.Input[str]]:
        """
        The port number. When the value of the Type attribute is 3, the PortStr attribute is required. Value range: **1-65535**.
        """
        return pulumi.get(self, "port_str")

    @port_str.setter
    def port_str(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "port_str", value)

    @property
    @pulumi.getter(name="procPath")
    def proc_path(self) -> Optional[pulumi.Input[str]]:
        """
        The process path. When the Type attribute is set to 2, 3, 4, 5, 6, or 7, the ProcPath attribute is required.
        """
        return pulumi.get(self, "proc_path")

    @proc_path.setter
    def proc_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "proc_path", value)

    @property
    @pulumi.getter(name="registryContent")
    def registry_content(self) -> Optional[pulumi.Input[str]]:
        """
        The registry value. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        """
        return pulumi.get(self, "registry_content")

    @registry_content.setter
    def registry_content(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "registry_content", value)

    @property
    @pulumi.getter(name="registryKey")
    def registry_key(self) -> Optional[pulumi.Input[str]]:
        """
        The registry key. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        """
        return pulumi.get(self, "registry_key")

    @registry_key.setter
    def registry_key(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "registry_key", value)


@pulumi.input_type
class _ClientUserDefineRuleState:
    def __init__(__self__, *,
                 action_type: Optional[pulumi.Input[int]] = None,
                 client_user_define_rule_name: Optional[pulumi.Input[str]] = None,
                 cmdline: Optional[pulumi.Input[str]] = None,
                 create_time: Optional[pulumi.Input[int]] = None,
                 file_path: Optional[pulumi.Input[str]] = None,
                 hash: Optional[pulumi.Input[str]] = None,
                 ip: Optional[pulumi.Input[str]] = None,
                 new_file_path: Optional[pulumi.Input[str]] = None,
                 parent_cmdline: Optional[pulumi.Input[str]] = None,
                 parent_proc_path: Optional[pulumi.Input[str]] = None,
                 platform: Optional[pulumi.Input[str]] = None,
                 port_str: Optional[pulumi.Input[str]] = None,
                 proc_path: Optional[pulumi.Input[str]] = None,
                 registry_content: Optional[pulumi.Input[str]] = None,
                 registry_key: Optional[pulumi.Input[str]] = None,
                 type: Optional[pulumi.Input[int]] = None):
        """
        Input properties used for looking up and filtering ClientUserDefineRule resources.
        :param pulumi.Input[int] action_type: The operation type. Value:
               - **0**: plus White
               - **1**: Plus Black.
        :param pulumi.Input[str] client_user_define_rule_name: The custom rule name.
        :param pulumi.Input[str] cmdline: Command line. When the value of the Type attribute is 2, 3, 4, 5, 6, or 7, the command line field is required.
        :param pulumi.Input[int] create_time: The creation time of the resource.
        :param pulumi.Input[str] file_path: The file path. When the value of the Type attribute is 4 or 6, 7, the FilePath field is required.
        :param pulumi.Input[str] hash: Process hash list. When the value of the Type attribute is 1, the Hash attribute is required.
        :param pulumi.Input[str] ip: IP address. When the value of the Type attribute is 3, the Ip attribute is required.
        :param pulumi.Input[str] new_file_path: The new file path to rename the file. When the value of the Type attribute is 7, the NewFilePath attribute is required.
        :param pulumi.Input[str] parent_cmdline: The parent command line.
        :param pulumi.Input[str] parent_proc_path: Parent process path.
        :param pulumi.Input[str] platform: The operating system type. Value:
               - **windows**:widows
               - **linux**:linux
               - **all**: all.
        :param pulumi.Input[str] port_str: The port number. When the value of the Type attribute is 3, the PortStr attribute is required. Value range: **1-65535**.
        :param pulumi.Input[str] proc_path: The process path. When the Type attribute is set to 2, 3, 4, 5, 6, or 7, the ProcPath attribute is required.
        :param pulumi.Input[str] registry_content: The registry value. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        :param pulumi.Input[str] registry_key: The registry key. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        :param pulumi.Input[int] type: The rule type. Value:
               - **1**: Process hash
               - **2**: command line
               - **3**: Process network
               - **4**: File reading and writing
               - **5**: Operate the registry
               - **6**: Load Dynamic Link Library
               - **7**: File Rename.
        """
        if action_type is not None:
            pulumi.set(__self__, "action_type", action_type)
        if client_user_define_rule_name is not None:
            pulumi.set(__self__, "client_user_define_rule_name", client_user_define_rule_name)
        if cmdline is not None:
            pulumi.set(__self__, "cmdline", cmdline)
        if create_time is not None:
            pulumi.set(__self__, "create_time", create_time)
        if file_path is not None:
            pulumi.set(__self__, "file_path", file_path)
        if hash is not None:
            pulumi.set(__self__, "hash", hash)
        if ip is not None:
            pulumi.set(__self__, "ip", ip)
        if new_file_path is not None:
            pulumi.set(__self__, "new_file_path", new_file_path)
        if parent_cmdline is not None:
            pulumi.set(__self__, "parent_cmdline", parent_cmdline)
        if parent_proc_path is not None:
            pulumi.set(__self__, "parent_proc_path", parent_proc_path)
        if platform is not None:
            pulumi.set(__self__, "platform", platform)
        if port_str is not None:
            pulumi.set(__self__, "port_str", port_str)
        if proc_path is not None:
            pulumi.set(__self__, "proc_path", proc_path)
        if registry_content is not None:
            pulumi.set(__self__, "registry_content", registry_content)
        if registry_key is not None:
            pulumi.set(__self__, "registry_key", registry_key)
        if type is not None:
            pulumi.set(__self__, "type", type)

    @property
    @pulumi.getter(name="actionType")
    def action_type(self) -> Optional[pulumi.Input[int]]:
        """
        The operation type. Value:
        - **0**: plus White
        - **1**: Plus Black.
        """
        return pulumi.get(self, "action_type")

    @action_type.setter
    def action_type(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "action_type", value)

    @property
    @pulumi.getter(name="clientUserDefineRuleName")
    def client_user_define_rule_name(self) -> Optional[pulumi.Input[str]]:
        """
        The custom rule name.
        """
        return pulumi.get(self, "client_user_define_rule_name")

    @client_user_define_rule_name.setter
    def client_user_define_rule_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "client_user_define_rule_name", value)

    @property
    @pulumi.getter
    def cmdline(self) -> Optional[pulumi.Input[str]]:
        """
        Command line. When the value of the Type attribute is 2, 3, 4, 5, 6, or 7, the command line field is required.
        """
        return pulumi.get(self, "cmdline")

    @cmdline.setter
    def cmdline(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "cmdline", value)

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> Optional[pulumi.Input[int]]:
        """
        The creation time of the resource.
        """
        return pulumi.get(self, "create_time")

    @create_time.setter
    def create_time(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "create_time", value)

    @property
    @pulumi.getter(name="filePath")
    def file_path(self) -> Optional[pulumi.Input[str]]:
        """
        The file path. When the value of the Type attribute is 4 or 6, 7, the FilePath field is required.
        """
        return pulumi.get(self, "file_path")

    @file_path.setter
    def file_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "file_path", value)

    @property
    @pulumi.getter
    def hash(self) -> Optional[pulumi.Input[str]]:
        """
        Process hash list. When the value of the Type attribute is 1, the Hash attribute is required.
        """
        return pulumi.get(self, "hash")

    @hash.setter
    def hash(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "hash", value)

    @property
    @pulumi.getter
    def ip(self) -> Optional[pulumi.Input[str]]:
        """
        IP address. When the value of the Type attribute is 3, the Ip attribute is required.
        """
        return pulumi.get(self, "ip")

    @ip.setter
    def ip(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ip", value)

    @property
    @pulumi.getter(name="newFilePath")
    def new_file_path(self) -> Optional[pulumi.Input[str]]:
        """
        The new file path to rename the file. When the value of the Type attribute is 7, the NewFilePath attribute is required.
        """
        return pulumi.get(self, "new_file_path")

    @new_file_path.setter
    def new_file_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "new_file_path", value)

    @property
    @pulumi.getter(name="parentCmdline")
    def parent_cmdline(self) -> Optional[pulumi.Input[str]]:
        """
        The parent command line.
        """
        return pulumi.get(self, "parent_cmdline")

    @parent_cmdline.setter
    def parent_cmdline(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "parent_cmdline", value)

    @property
    @pulumi.getter(name="parentProcPath")
    def parent_proc_path(self) -> Optional[pulumi.Input[str]]:
        """
        Parent process path.
        """
        return pulumi.get(self, "parent_proc_path")

    @parent_proc_path.setter
    def parent_proc_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "parent_proc_path", value)

    @property
    @pulumi.getter
    def platform(self) -> Optional[pulumi.Input[str]]:
        """
        The operating system type. Value:
        - **windows**:widows
        - **linux**:linux
        - **all**: all.
        """
        return pulumi.get(self, "platform")

    @platform.setter
    def platform(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "platform", value)

    @property
    @pulumi.getter(name="portStr")
    def port_str(self) -> Optional[pulumi.Input[str]]:
        """
        The port number. When the value of the Type attribute is 3, the PortStr attribute is required. Value range: **1-65535**.
        """
        return pulumi.get(self, "port_str")

    @port_str.setter
    def port_str(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "port_str", value)

    @property
    @pulumi.getter(name="procPath")
    def proc_path(self) -> Optional[pulumi.Input[str]]:
        """
        The process path. When the Type attribute is set to 2, 3, 4, 5, 6, or 7, the ProcPath attribute is required.
        """
        return pulumi.get(self, "proc_path")

    @proc_path.setter
    def proc_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "proc_path", value)

    @property
    @pulumi.getter(name="registryContent")
    def registry_content(self) -> Optional[pulumi.Input[str]]:
        """
        The registry value. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        """
        return pulumi.get(self, "registry_content")

    @registry_content.setter
    def registry_content(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "registry_content", value)

    @property
    @pulumi.getter(name="registryKey")
    def registry_key(self) -> Optional[pulumi.Input[str]]:
        """
        The registry key. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        """
        return pulumi.get(self, "registry_key")

    @registry_key.setter
    def registry_key(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "registry_key", value)

    @property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[int]]:
        """
        The rule type. Value:
        - **1**: Process hash
        - **2**: command line
        - **3**: Process network
        - **4**: File reading and writing
        - **5**: Operate the registry
        - **6**: Load Dynamic Link Library
        - **7**: File Rename.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "type", value)


class ClientUserDefineRule(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 action_type: Optional[pulumi.Input[int]] = None,
                 client_user_define_rule_name: Optional[pulumi.Input[str]] = None,
                 cmdline: Optional[pulumi.Input[str]] = None,
                 file_path: Optional[pulumi.Input[str]] = None,
                 hash: Optional[pulumi.Input[str]] = None,
                 ip: Optional[pulumi.Input[str]] = None,
                 new_file_path: Optional[pulumi.Input[str]] = None,
                 parent_cmdline: Optional[pulumi.Input[str]] = None,
                 parent_proc_path: Optional[pulumi.Input[str]] = None,
                 platform: Optional[pulumi.Input[str]] = None,
                 port_str: Optional[pulumi.Input[str]] = None,
                 proc_path: Optional[pulumi.Input[str]] = None,
                 registry_content: Optional[pulumi.Input[str]] = None,
                 registry_key: Optional[pulumi.Input[str]] = None,
                 type: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        """
        Provides a Threat Detection Client User Define Rule resource. Malicious Behavior Defense Custom Rules.

        For information about Threat Detection Client User Define Rule and how to use it, see [What is Client User Define Rule](https://www.alibabacloud.com/help/en/security-center/developer-reference/api-sas-2018-12-03-addclientuserdefinerule).

        > **NOTE:** Available since v1.212.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "terraform-example"
        default = alicloud.threatdetection.ClientUserDefineRule("default",
            action_type=0,
            platform="windows",
            registry_content="123",
            client_user_define_rule_name=name,
            parent_proc_path="/root/bash",
            type=5,
            cmdline="bash",
            proc_path="/root/bash",
            parent_cmdline="bash",
            registry_key="123")
        ```

        ## Import

        Threat Detection Client User Define Rule can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:threatdetection/clientUserDefineRule:ClientUserDefineRule example <id>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[int] action_type: The operation type. Value:
               - **0**: plus White
               - **1**: Plus Black.
        :param pulumi.Input[str] client_user_define_rule_name: The custom rule name.
        :param pulumi.Input[str] cmdline: Command line. When the value of the Type attribute is 2, 3, 4, 5, 6, or 7, the command line field is required.
        :param pulumi.Input[str] file_path: The file path. When the value of the Type attribute is 4 or 6, 7, the FilePath field is required.
        :param pulumi.Input[str] hash: Process hash list. When the value of the Type attribute is 1, the Hash attribute is required.
        :param pulumi.Input[str] ip: IP address. When the value of the Type attribute is 3, the Ip attribute is required.
        :param pulumi.Input[str] new_file_path: The new file path to rename the file. When the value of the Type attribute is 7, the NewFilePath attribute is required.
        :param pulumi.Input[str] parent_cmdline: The parent command line.
        :param pulumi.Input[str] parent_proc_path: Parent process path.
        :param pulumi.Input[str] platform: The operating system type. Value:
               - **windows**:widows
               - **linux**:linux
               - **all**: all.
        :param pulumi.Input[str] port_str: The port number. When the value of the Type attribute is 3, the PortStr attribute is required. Value range: **1-65535**.
        :param pulumi.Input[str] proc_path: The process path. When the Type attribute is set to 2, 3, 4, 5, 6, or 7, the ProcPath attribute is required.
        :param pulumi.Input[str] registry_content: The registry value. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        :param pulumi.Input[str] registry_key: The registry key. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        :param pulumi.Input[int] type: The rule type. Value:
               - **1**: Process hash
               - **2**: command line
               - **3**: Process network
               - **4**: File reading and writing
               - **5**: Operate the registry
               - **6**: Load Dynamic Link Library
               - **7**: File Rename.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ClientUserDefineRuleArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Threat Detection Client User Define Rule resource. Malicious Behavior Defense Custom Rules.

        For information about Threat Detection Client User Define Rule and how to use it, see [What is Client User Define Rule](https://www.alibabacloud.com/help/en/security-center/developer-reference/api-sas-2018-12-03-addclientuserdefinerule).

        > **NOTE:** Available since v1.212.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "terraform-example"
        default = alicloud.threatdetection.ClientUserDefineRule("default",
            action_type=0,
            platform="windows",
            registry_content="123",
            client_user_define_rule_name=name,
            parent_proc_path="/root/bash",
            type=5,
            cmdline="bash",
            proc_path="/root/bash",
            parent_cmdline="bash",
            registry_key="123")
        ```

        ## Import

        Threat Detection Client User Define Rule can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:threatdetection/clientUserDefineRule:ClientUserDefineRule example <id>
        ```

        :param str resource_name: The name of the resource.
        :param ClientUserDefineRuleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ClientUserDefineRuleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 action_type: Optional[pulumi.Input[int]] = None,
                 client_user_define_rule_name: Optional[pulumi.Input[str]] = None,
                 cmdline: Optional[pulumi.Input[str]] = None,
                 file_path: Optional[pulumi.Input[str]] = None,
                 hash: Optional[pulumi.Input[str]] = None,
                 ip: Optional[pulumi.Input[str]] = None,
                 new_file_path: Optional[pulumi.Input[str]] = None,
                 parent_cmdline: Optional[pulumi.Input[str]] = None,
                 parent_proc_path: Optional[pulumi.Input[str]] = None,
                 platform: Optional[pulumi.Input[str]] = None,
                 port_str: Optional[pulumi.Input[str]] = None,
                 proc_path: Optional[pulumi.Input[str]] = None,
                 registry_content: Optional[pulumi.Input[str]] = None,
                 registry_key: Optional[pulumi.Input[str]] = None,
                 type: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ClientUserDefineRuleArgs.__new__(ClientUserDefineRuleArgs)

            if action_type is None and not opts.urn:
                raise TypeError("Missing required property 'action_type'")
            __props__.__dict__["action_type"] = action_type
            if client_user_define_rule_name is None and not opts.urn:
                raise TypeError("Missing required property 'client_user_define_rule_name'")
            __props__.__dict__["client_user_define_rule_name"] = client_user_define_rule_name
            __props__.__dict__["cmdline"] = cmdline
            __props__.__dict__["file_path"] = file_path
            __props__.__dict__["hash"] = hash
            __props__.__dict__["ip"] = ip
            __props__.__dict__["new_file_path"] = new_file_path
            __props__.__dict__["parent_cmdline"] = parent_cmdline
            __props__.__dict__["parent_proc_path"] = parent_proc_path
            if platform is None and not opts.urn:
                raise TypeError("Missing required property 'platform'")
            __props__.__dict__["platform"] = platform
            __props__.__dict__["port_str"] = port_str
            __props__.__dict__["proc_path"] = proc_path
            __props__.__dict__["registry_content"] = registry_content
            __props__.__dict__["registry_key"] = registry_key
            if type is None and not opts.urn:
                raise TypeError("Missing required property 'type'")
            __props__.__dict__["type"] = type
            __props__.__dict__["create_time"] = None
        super(ClientUserDefineRule, __self__).__init__(
            'alicloud:threatdetection/clientUserDefineRule:ClientUserDefineRule',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            action_type: Optional[pulumi.Input[int]] = None,
            client_user_define_rule_name: Optional[pulumi.Input[str]] = None,
            cmdline: Optional[pulumi.Input[str]] = None,
            create_time: Optional[pulumi.Input[int]] = None,
            file_path: Optional[pulumi.Input[str]] = None,
            hash: Optional[pulumi.Input[str]] = None,
            ip: Optional[pulumi.Input[str]] = None,
            new_file_path: Optional[pulumi.Input[str]] = None,
            parent_cmdline: Optional[pulumi.Input[str]] = None,
            parent_proc_path: Optional[pulumi.Input[str]] = None,
            platform: Optional[pulumi.Input[str]] = None,
            port_str: Optional[pulumi.Input[str]] = None,
            proc_path: Optional[pulumi.Input[str]] = None,
            registry_content: Optional[pulumi.Input[str]] = None,
            registry_key: Optional[pulumi.Input[str]] = None,
            type: Optional[pulumi.Input[int]] = None) -> 'ClientUserDefineRule':
        """
        Get an existing ClientUserDefineRule resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[int] action_type: The operation type. Value:
               - **0**: plus White
               - **1**: Plus Black.
        :param pulumi.Input[str] client_user_define_rule_name: The custom rule name.
        :param pulumi.Input[str] cmdline: Command line. When the value of the Type attribute is 2, 3, 4, 5, 6, or 7, the command line field is required.
        :param pulumi.Input[int] create_time: The creation time of the resource.
        :param pulumi.Input[str] file_path: The file path. When the value of the Type attribute is 4 or 6, 7, the FilePath field is required.
        :param pulumi.Input[str] hash: Process hash list. When the value of the Type attribute is 1, the Hash attribute is required.
        :param pulumi.Input[str] ip: IP address. When the value of the Type attribute is 3, the Ip attribute is required.
        :param pulumi.Input[str] new_file_path: The new file path to rename the file. When the value of the Type attribute is 7, the NewFilePath attribute is required.
        :param pulumi.Input[str] parent_cmdline: The parent command line.
        :param pulumi.Input[str] parent_proc_path: Parent process path.
        :param pulumi.Input[str] platform: The operating system type. Value:
               - **windows**:widows
               - **linux**:linux
               - **all**: all.
        :param pulumi.Input[str] port_str: The port number. When the value of the Type attribute is 3, the PortStr attribute is required. Value range: **1-65535**.
        :param pulumi.Input[str] proc_path: The process path. When the Type attribute is set to 2, 3, 4, 5, 6, or 7, the ProcPath attribute is required.
        :param pulumi.Input[str] registry_content: The registry value. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        :param pulumi.Input[str] registry_key: The registry key. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        :param pulumi.Input[int] type: The rule type. Value:
               - **1**: Process hash
               - **2**: command line
               - **3**: Process network
               - **4**: File reading and writing
               - **5**: Operate the registry
               - **6**: Load Dynamic Link Library
               - **7**: File Rename.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ClientUserDefineRuleState.__new__(_ClientUserDefineRuleState)

        __props__.__dict__["action_type"] = action_type
        __props__.__dict__["client_user_define_rule_name"] = client_user_define_rule_name
        __props__.__dict__["cmdline"] = cmdline
        __props__.__dict__["create_time"] = create_time
        __props__.__dict__["file_path"] = file_path
        __props__.__dict__["hash"] = hash
        __props__.__dict__["ip"] = ip
        __props__.__dict__["new_file_path"] = new_file_path
        __props__.__dict__["parent_cmdline"] = parent_cmdline
        __props__.__dict__["parent_proc_path"] = parent_proc_path
        __props__.__dict__["platform"] = platform
        __props__.__dict__["port_str"] = port_str
        __props__.__dict__["proc_path"] = proc_path
        __props__.__dict__["registry_content"] = registry_content
        __props__.__dict__["registry_key"] = registry_key
        __props__.__dict__["type"] = type
        return ClientUserDefineRule(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="actionType")
    def action_type(self) -> pulumi.Output[int]:
        """
        The operation type. Value:
        - **0**: plus White
        - **1**: Plus Black.
        """
        return pulumi.get(self, "action_type")

    @property
    @pulumi.getter(name="clientUserDefineRuleName")
    def client_user_define_rule_name(self) -> pulumi.Output[str]:
        """
        The custom rule name.
        """
        return pulumi.get(self, "client_user_define_rule_name")

    @property
    @pulumi.getter
    def cmdline(self) -> pulumi.Output[Optional[str]]:
        """
        Command line. When the value of the Type attribute is 2, 3, 4, 5, 6, or 7, the command line field is required.
        """
        return pulumi.get(self, "cmdline")

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> pulumi.Output[int]:
        """
        The creation time of the resource.
        """
        return pulumi.get(self, "create_time")

    @property
    @pulumi.getter(name="filePath")
    def file_path(self) -> pulumi.Output[Optional[str]]:
        """
        The file path. When the value of the Type attribute is 4 or 6, 7, the FilePath field is required.
        """
        return pulumi.get(self, "file_path")

    @property
    @pulumi.getter
    def hash(self) -> pulumi.Output[Optional[str]]:
        """
        Process hash list. When the value of the Type attribute is 1, the Hash attribute is required.
        """
        return pulumi.get(self, "hash")

    @property
    @pulumi.getter
    def ip(self) -> pulumi.Output[Optional[str]]:
        """
        IP address. When the value of the Type attribute is 3, the Ip attribute is required.
        """
        return pulumi.get(self, "ip")

    @property
    @pulumi.getter(name="newFilePath")
    def new_file_path(self) -> pulumi.Output[Optional[str]]:
        """
        The new file path to rename the file. When the value of the Type attribute is 7, the NewFilePath attribute is required.
        """
        return pulumi.get(self, "new_file_path")

    @property
    @pulumi.getter(name="parentCmdline")
    def parent_cmdline(self) -> pulumi.Output[Optional[str]]:
        """
        The parent command line.
        """
        return pulumi.get(self, "parent_cmdline")

    @property
    @pulumi.getter(name="parentProcPath")
    def parent_proc_path(self) -> pulumi.Output[Optional[str]]:
        """
        Parent process path.
        """
        return pulumi.get(self, "parent_proc_path")

    @property
    @pulumi.getter
    def platform(self) -> pulumi.Output[str]:
        """
        The operating system type. Value:
        - **windows**:widows
        - **linux**:linux
        - **all**: all.
        """
        return pulumi.get(self, "platform")

    @property
    @pulumi.getter(name="portStr")
    def port_str(self) -> pulumi.Output[str]:
        """
        The port number. When the value of the Type attribute is 3, the PortStr attribute is required. Value range: **1-65535**.
        """
        return pulumi.get(self, "port_str")

    @property
    @pulumi.getter(name="procPath")
    def proc_path(self) -> pulumi.Output[Optional[str]]:
        """
        The process path. When the Type attribute is set to 2, 3, 4, 5, 6, or 7, the ProcPath attribute is required.
        """
        return pulumi.get(self, "proc_path")

    @property
    @pulumi.getter(name="registryContent")
    def registry_content(self) -> pulumi.Output[Optional[str]]:
        """
        The registry value. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        """
        return pulumi.get(self, "registry_content")

    @property
    @pulumi.getter(name="registryKey")
    def registry_key(self) -> pulumi.Output[Optional[str]]:
        """
        The registry key. When the value of the Type attribute is 5, the RegistryKey attribute is required.
        """
        return pulumi.get(self, "registry_key")

    @property
    @pulumi.getter
    def type(self) -> pulumi.Output[int]:
        """
        The rule type. Value:
        - **1**: Process hash
        - **2**: command line
        - **3**: Process network
        - **4**: File reading and writing
        - **5**: Operate the registry
        - **6**: Load Dynamic Link Library
        - **7**: File Rename.
        """
        return pulumi.get(self, "type")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs

__all__ = [
    'DbClusterDesiredParam',
    'DbClusterParamChangeLog',
    'DbInstanceDesiredSecurityIpList',
    'DbInstanceInstanceNetInfo',
    'DbInstanceInstanceNetInfoPortList',
    'DbInstanceSecurityIpList',
    'GetDbClustersClusterResult',
    'GetDbClustersClusterParamResult',
    'GetDbClustersClusterParamChangeLogResult',
    'GetDbInstancesInstanceResult',
]

@pulumi.output_type
class DbClusterDesiredParam(dict):
    def __init__(__self__, *,
                 name: Optional[str] = None,
                 value: Optional[str] = None):
        """
        :param str name: Parameter name.
        :param str value: The new value of Parameter.
        """
        if name is not None:
            pulumi.set(__self__, "name", name)
        if value is not None:
            pulumi.set(__self__, "value", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[str]:
        """
        Parameter name.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def value(self) -> Optional[str]:
        """
        The new value of Parameter.
        """
        return pulumi.get(self, "value")


@pulumi.output_type
class DbClusterParamChangeLog(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "configId":
            suggest = "config_id"
        elif key == "gmtCreated":
            suggest = "gmt_created"
        elif key == "gmtModified":
            suggest = "gmt_modified"
        elif key == "isApplied":
            suggest = "is_applied"
        elif key == "newValue":
            suggest = "new_value"
        elif key == "oldValue":
            suggest = "old_value"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DbClusterParamChangeLog. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DbClusterParamChangeLog.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DbClusterParamChangeLog.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 config_id: Optional[int] = None,
                 gmt_created: Optional[str] = None,
                 gmt_modified: Optional[str] = None,
                 is_applied: Optional[bool] = None,
                 name: Optional[str] = None,
                 new_value: Optional[str] = None,
                 old_value: Optional[str] = None):
        """
        :param int config_id: The id of parameter change.
        :param str gmt_created: When the parameter change is created.
        :param str gmt_modified: When the parameter change is modified.
        :param bool is_applied: Whether the parameter changing is applied.
        :param str name: Changed parameter name.
        :param str new_value: The new value of parameter.
        :param str old_value: The old value of parameter.
        """
        if config_id is not None:
            pulumi.set(__self__, "config_id", config_id)
        if gmt_created is not None:
            pulumi.set(__self__, "gmt_created", gmt_created)
        if gmt_modified is not None:
            pulumi.set(__self__, "gmt_modified", gmt_modified)
        if is_applied is not None:
            pulumi.set(__self__, "is_applied", is_applied)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if new_value is not None:
            pulumi.set(__self__, "new_value", new_value)
        if old_value is not None:
            pulumi.set(__self__, "old_value", old_value)

    @property
    @pulumi.getter(name="configId")
    def config_id(self) -> Optional[int]:
        """
        The id of parameter change.
        """
        return pulumi.get(self, "config_id")

    @property
    @pulumi.getter(name="gmtCreated")
    def gmt_created(self) -> Optional[str]:
        """
        When the parameter change is created.
        """
        return pulumi.get(self, "gmt_created")

    @property
    @pulumi.getter(name="gmtModified")
    def gmt_modified(self) -> Optional[str]:
        """
        When the parameter change is modified.
        """
        return pulumi.get(self, "gmt_modified")

    @property
    @pulumi.getter(name="isApplied")
    def is_applied(self) -> Optional[bool]:
        """
        Whether the parameter changing is applied.
        """
        return pulumi.get(self, "is_applied")

    @property
    @pulumi.getter
    def name(self) -> Optional[str]:
        """
        Changed parameter name.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="newValue")
    def new_value(self) -> Optional[str]:
        """
        The new value of parameter.
        """
        return pulumi.get(self, "new_value")

    @property
    @pulumi.getter(name="oldValue")
    def old_value(self) -> Optional[str]:
        """
        The old value of parameter.
        """
        return pulumi.get(self, "old_value")


@pulumi.output_type
class DbInstanceDesiredSecurityIpList(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "groupName":
            suggest = "group_name"
        elif key == "securityIpList":
            suggest = "security_ip_list"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DbInstanceDesiredSecurityIpList. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DbInstanceDesiredSecurityIpList.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DbInstanceDesiredSecurityIpList.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 group_name: Optional[str] = None,
                 security_ip_list: Optional[str] = None):
        """
        :param str group_name: Security group name.
        :param str security_ip_list: The IP list of Security group. Each single IP value should be Separated by comma.
        """
        if group_name is not None:
            pulumi.set(__self__, "group_name", group_name)
        if security_ip_list is not None:
            pulumi.set(__self__, "security_ip_list", security_ip_list)

    @property
    @pulumi.getter(name="groupName")
    def group_name(self) -> Optional[str]:
        """
        Security group name.
        """
        return pulumi.get(self, "group_name")

    @property
    @pulumi.getter(name="securityIpList")
    def security_ip_list(self) -> Optional[str]:
        """
        The IP list of Security group. Each single IP value should be Separated by comma.
        """
        return pulumi.get(self, "security_ip_list")


@pulumi.output_type
class DbInstanceInstanceNetInfo(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "connectionString":
            suggest = "connection_string"
        elif key == "dbIp":
            suggest = "db_ip"
        elif key == "netType":
            suggest = "net_type"
        elif key == "portLists":
            suggest = "port_lists"
        elif key == "vpcInstanceId":
            suggest = "vpc_instance_id"
        elif key == "vswitchId":
            suggest = "vswitch_id"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DbInstanceInstanceNetInfo. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DbInstanceInstanceNetInfo.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DbInstanceInstanceNetInfo.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 connection_string: Optional[str] = None,
                 db_ip: Optional[str] = None,
                 net_type: Optional[str] = None,
                 port_lists: Optional[Sequence['outputs.DbInstanceInstanceNetInfoPortList']] = None,
                 vpc_instance_id: Optional[str] = None,
                 vswitch_id: Optional[str] = None):
        """
        :param str connection_string: The connection string of the instance.
        :param str db_ip: The IP address of the instance.
        :param str net_type: The network type of the instance.
        :param Sequence['DbInstanceInstanceNetInfoPortListArgs'] port_lists: A list for port provides SelectDB service.
        :param str vpc_instance_id: The VPC ID.
        :param str vswitch_id: The ID of vswitch for DBInstance.
        """
        if connection_string is not None:
            pulumi.set(__self__, "connection_string", connection_string)
        if db_ip is not None:
            pulumi.set(__self__, "db_ip", db_ip)
        if net_type is not None:
            pulumi.set(__self__, "net_type", net_type)
        if port_lists is not None:
            pulumi.set(__self__, "port_lists", port_lists)
        if vpc_instance_id is not None:
            pulumi.set(__self__, "vpc_instance_id", vpc_instance_id)
        if vswitch_id is not None:
            pulumi.set(__self__, "vswitch_id", vswitch_id)

    @property
    @pulumi.getter(name="connectionString")
    def connection_string(self) -> Optional[str]:
        """
        The connection string of the instance.
        """
        return pulumi.get(self, "connection_string")

    @property
    @pulumi.getter(name="dbIp")
    def db_ip(self) -> Optional[str]:
        """
        The IP address of the instance.
        """
        return pulumi.get(self, "db_ip")

    @property
    @pulumi.getter(name="netType")
    def net_type(self) -> Optional[str]:
        """
        The network type of the instance.
        """
        return pulumi.get(self, "net_type")

    @property
    @pulumi.getter(name="portLists")
    def port_lists(self) -> Optional[Sequence['outputs.DbInstanceInstanceNetInfoPortList']]:
        """
        A list for port provides SelectDB service.
        """
        return pulumi.get(self, "port_lists")

    @property
    @pulumi.getter(name="vpcInstanceId")
    def vpc_instance_id(self) -> Optional[str]:
        """
        The VPC ID.
        """
        return pulumi.get(self, "vpc_instance_id")

    @property
    @pulumi.getter(name="vswitchId")
    def vswitch_id(self) -> Optional[str]:
        """
        The ID of vswitch for DBInstance.
        """
        return pulumi.get(self, "vswitch_id")


@pulumi.output_type
class DbInstanceInstanceNetInfoPortList(dict):
    def __init__(__self__, *,
                 port: Optional[str] = None,
                 protocol: Optional[str] = None):
        """
        :param str port: The port that is used to connect.
        :param str protocol: The protocol of the port.
        """
        if port is not None:
            pulumi.set(__self__, "port", port)
        if protocol is not None:
            pulumi.set(__self__, "protocol", protocol)

    @property
    @pulumi.getter
    def port(self) -> Optional[str]:
        """
        The port that is used to connect.
        """
        return pulumi.get(self, "port")

    @property
    @pulumi.getter
    def protocol(self) -> Optional[str]:
        """
        The protocol of the port.
        """
        return pulumi.get(self, "protocol")


@pulumi.output_type
class DbInstanceSecurityIpList(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "groupName":
            suggest = "group_name"
        elif key == "groupTag":
            suggest = "group_tag"
        elif key == "listNetType":
            suggest = "list_net_type"
        elif key == "securityIpList":
            suggest = "security_ip_list"
        elif key == "securityIpType":
            suggest = "security_ip_type"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DbInstanceSecurityIpList. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DbInstanceSecurityIpList.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DbInstanceSecurityIpList.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 group_name: Optional[str] = None,
                 group_tag: Optional[str] = None,
                 list_net_type: Optional[str] = None,
                 security_ip_list: Optional[str] = None,
                 security_ip_type: Optional[str] = None):
        """
        :param str group_name: Security group name.
        :param str group_tag: The tag of Security group.
        :param str list_net_type: The network type of Security group.
        :param str security_ip_list: The IP list of Security group. Each single IP value should be Separated by comma.
        :param str security_ip_type: The IP address type. Valid values: `ipv4`, `ipv6` (not supported).
        """
        if group_name is not None:
            pulumi.set(__self__, "group_name", group_name)
        if group_tag is not None:
            pulumi.set(__self__, "group_tag", group_tag)
        if list_net_type is not None:
            pulumi.set(__self__, "list_net_type", list_net_type)
        if security_ip_list is not None:
            pulumi.set(__self__, "security_ip_list", security_ip_list)
        if security_ip_type is not None:
            pulumi.set(__self__, "security_ip_type", security_ip_type)

    @property
    @pulumi.getter(name="groupName")
    def group_name(self) -> Optional[str]:
        """
        Security group name.
        """
        return pulumi.get(self, "group_name")

    @property
    @pulumi.getter(name="groupTag")
    def group_tag(self) -> Optional[str]:
        """
        The tag of Security group.
        """
        return pulumi.get(self, "group_tag")

    @property
    @pulumi.getter(name="listNetType")
    def list_net_type(self) -> Optional[str]:
        """
        The network type of Security group.
        """
        return pulumi.get(self, "list_net_type")

    @property
    @pulumi.getter(name="securityIpList")
    def security_ip_list(self) -> Optional[str]:
        """
        The IP list of Security group. Each single IP value should be Separated by comma.
        """
        return pulumi.get(self, "security_ip_list")

    @property
    @pulumi.getter(name="securityIpType")
    def security_ip_type(self) -> Optional[str]:
        """
        The IP address type. Valid values: `ipv4`, `ipv6` (not supported).
        """
        return pulumi.get(self, "security_ip_type")


@pulumi.output_type
class GetDbClustersClusterResult(dict):
    def __init__(__self__, *,
                 cache_size: int,
                 cpu: int,
                 create_time: str,
                 db_cluster_class: str,
                 db_cluster_description: str,
                 db_cluster_id: str,
                 db_instance_id: str,
                 engine: str,
                 engine_version: str,
                 id: str,
                 memory: int,
                 param_change_logs: Sequence['outputs.GetDbClustersClusterParamChangeLogResult'],
                 params: Sequence['outputs.GetDbClustersClusterParamResult'],
                 payment_type: str,
                 region_id: str,
                 status: str,
                 vpc_id: str,
                 zone_id: str):
        """
        :param int cache_size: The cache size for DBCluster.
        :param int cpu: The cpu resource amount of DBCluster. Depends on `db_cluster_class`.
        :param str create_time: The creation time of the resource.
        :param str db_cluster_class: The DBCluster class. db_cluster_class has a range of class from `selectdb.xlarge` to `selectdb.256xlarge`.
        :param str db_cluster_description: The DBCluster description.
        :param str db_cluster_id: The cluster ID.
        :param str db_instance_id: The instance ID.
        :param str engine: The Engine of the DBCluster.
        :param str engine_version: The engine version of the DBCluster.
        :param int memory: The memory resource amount of DBCluster. Depends on `db_cluster_class`.
        :param Sequence['GetDbClustersClusterParamChangeLogArgs'] param_change_logs: The configuration change logs of parameters.
        :param Sequence['GetDbClustersClusterParamArgs'] params: The details about each parameter in DBCluster returned.
        :param str payment_type: The payment type of the resource. Valid values: `PayAsYouGo`,`Subscription`.
        :param str region_id: The ID of region for the cluster.
        :param str status: The status of the DBCluster. Valid values: `ACTIVATION`,`CREATING`,`DELETING`,`RESTARTING`,`ORDER_PREPARING`.
        :param str vpc_id: The ID of the VPC for the cluster.
        :param str zone_id: The ID of zone for the cluster.
        """
        pulumi.set(__self__, "cache_size", cache_size)
        pulumi.set(__self__, "cpu", cpu)
        pulumi.set(__self__, "create_time", create_time)
        pulumi.set(__self__, "db_cluster_class", db_cluster_class)
        pulumi.set(__self__, "db_cluster_description", db_cluster_description)
        pulumi.set(__self__, "db_cluster_id", db_cluster_id)
        pulumi.set(__self__, "db_instance_id", db_instance_id)
        pulumi.set(__self__, "engine", engine)
        pulumi.set(__self__, "engine_version", engine_version)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "memory", memory)
        pulumi.set(__self__, "param_change_logs", param_change_logs)
        pulumi.set(__self__, "params", params)
        pulumi.set(__self__, "payment_type", payment_type)
        pulumi.set(__self__, "region_id", region_id)
        pulumi.set(__self__, "status", status)
        pulumi.set(__self__, "vpc_id", vpc_id)
        pulumi.set(__self__, "zone_id", zone_id)

    @property
    @pulumi.getter(name="cacheSize")
    def cache_size(self) -> int:
        """
        The cache size for DBCluster.
        """
        return pulumi.get(self, "cache_size")

    @property
    @pulumi.getter
    def cpu(self) -> int:
        """
        The cpu resource amount of DBCluster. Depends on `db_cluster_class`.
        """
        return pulumi.get(self, "cpu")

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> str:
        """
        The creation time of the resource.
        """
        return pulumi.get(self, "create_time")

    @property
    @pulumi.getter(name="dbClusterClass")
    def db_cluster_class(self) -> str:
        """
        The DBCluster class. db_cluster_class has a range of class from `selectdb.xlarge` to `selectdb.256xlarge`.
        """
        return pulumi.get(self, "db_cluster_class")

    @property
    @pulumi.getter(name="dbClusterDescription")
    def db_cluster_description(self) -> str:
        """
        The DBCluster description.
        """
        return pulumi.get(self, "db_cluster_description")

    @property
    @pulumi.getter(name="dbClusterId")
    def db_cluster_id(self) -> str:
        """
        The cluster ID.
        """
        return pulumi.get(self, "db_cluster_id")

    @property
    @pulumi.getter(name="dbInstanceId")
    def db_instance_id(self) -> str:
        """
        The instance ID.
        """
        return pulumi.get(self, "db_instance_id")

    @property
    @pulumi.getter
    def engine(self) -> str:
        """
        The Engine of the DBCluster.
        """
        return pulumi.get(self, "engine")

    @property
    @pulumi.getter(name="engineVersion")
    def engine_version(self) -> str:
        """
        The engine version of the DBCluster.
        """
        return pulumi.get(self, "engine_version")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def memory(self) -> int:
        """
        The memory resource amount of DBCluster. Depends on `db_cluster_class`.
        """
        return pulumi.get(self, "memory")

    @property
    @pulumi.getter(name="paramChangeLogs")
    def param_change_logs(self) -> Sequence['outputs.GetDbClustersClusterParamChangeLogResult']:
        """
        The configuration change logs of parameters.
        """
        return pulumi.get(self, "param_change_logs")

    @property
    @pulumi.getter
    def params(self) -> Sequence['outputs.GetDbClustersClusterParamResult']:
        """
        The details about each parameter in DBCluster returned.
        """
        return pulumi.get(self, "params")

    @property
    @pulumi.getter(name="paymentType")
    def payment_type(self) -> str:
        """
        The payment type of the resource. Valid values: `PayAsYouGo`,`Subscription`.
        """
        return pulumi.get(self, "payment_type")

    @property
    @pulumi.getter(name="regionId")
    def region_id(self) -> str:
        """
        The ID of region for the cluster.
        """
        return pulumi.get(self, "region_id")

    @property
    @pulumi.getter
    def status(self) -> str:
        """
        The status of the DBCluster. Valid values: `ACTIVATION`,`CREATING`,`DELETING`,`RESTARTING`,`ORDER_PREPARING`.
        """
        return pulumi.get(self, "status")

    @property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> str:
        """
        The ID of the VPC for the cluster.
        """
        return pulumi.get(self, "vpc_id")

    @property
    @pulumi.getter(name="zoneId")
    def zone_id(self) -> str:
        """
        The ID of zone for the cluster.
        """
        return pulumi.get(self, "zone_id")


@pulumi.output_type
class GetDbClustersClusterParamResult(dict):
    def __init__(__self__, *,
                 comment: str,
                 default_value: str,
                 is_dynamic: int,
                 is_user_modifiable: int,
                 name: str,
                 optional: int,
                 param_category: str,
                 value: str):
        """
        :param str comment: The comments on the parameter.
        :param str default_value: The default value of the parameter.
        :param int is_dynamic: Indicates whether the parameter immediately takes effect without requiring a restart.
        :param int is_user_modifiable: Indicates whether the parameter is modifiable.
        :param str name: Changed parameter name.
        :param int optional: The value range of the parameter.
        :param str param_category: The category of the parameter.
        :param str value: The new value of Parameter.
        """
        pulumi.set(__self__, "comment", comment)
        pulumi.set(__self__, "default_value", default_value)
        pulumi.set(__self__, "is_dynamic", is_dynamic)
        pulumi.set(__self__, "is_user_modifiable", is_user_modifiable)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "optional", optional)
        pulumi.set(__self__, "param_category", param_category)
        pulumi.set(__self__, "value", value)

    @property
    @pulumi.getter
    def comment(self) -> str:
        """
        The comments on the parameter.
        """
        return pulumi.get(self, "comment")

    @property
    @pulumi.getter(name="defaultValue")
    def default_value(self) -> str:
        """
        The default value of the parameter.
        """
        return pulumi.get(self, "default_value")

    @property
    @pulumi.getter(name="isDynamic")
    def is_dynamic(self) -> int:
        """
        Indicates whether the parameter immediately takes effect without requiring a restart.
        """
        return pulumi.get(self, "is_dynamic")

    @property
    @pulumi.getter(name="isUserModifiable")
    def is_user_modifiable(self) -> int:
        """
        Indicates whether the parameter is modifiable.
        """
        return pulumi.get(self, "is_user_modifiable")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        Changed parameter name.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def optional(self) -> int:
        """
        The value range of the parameter.
        """
        return pulumi.get(self, "optional")

    @property
    @pulumi.getter(name="paramCategory")
    def param_category(self) -> str:
        """
        The category of the parameter.
        """
        return pulumi.get(self, "param_category")

    @property
    @pulumi.getter
    def value(self) -> str:
        """
        The new value of Parameter.
        """
        return pulumi.get(self, "value")


@pulumi.output_type
class GetDbClustersClusterParamChangeLogResult(dict):
    def __init__(__self__, *,
                 config_id: int,
                 gmt_created: str,
                 gmt_modified: str,
                 is_applied: int,
                 name: str,
                 new_value: str,
                 old_value: str):
        """
        :param int config_id: The id of parameter change.
        :param str gmt_created: When the parameter change is created.
        :param str gmt_modified: When the parameter change is modified.
        :param int is_applied: Whether the parameter changing is applied.
        :param str name: Changed parameter name.
        :param str new_value: The new value of parameter.
        :param str old_value: The old value of parameter.
        """
        pulumi.set(__self__, "config_id", config_id)
        pulumi.set(__self__, "gmt_created", gmt_created)
        pulumi.set(__self__, "gmt_modified", gmt_modified)
        pulumi.set(__self__, "is_applied", is_applied)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "new_value", new_value)
        pulumi.set(__self__, "old_value", old_value)

    @property
    @pulumi.getter(name="configId")
    def config_id(self) -> int:
        """
        The id of parameter change.
        """
        return pulumi.get(self, "config_id")

    @property
    @pulumi.getter(name="gmtCreated")
    def gmt_created(self) -> str:
        """
        When the parameter change is created.
        """
        return pulumi.get(self, "gmt_created")

    @property
    @pulumi.getter(name="gmtModified")
    def gmt_modified(self) -> str:
        """
        When the parameter change is modified.
        """
        return pulumi.get(self, "gmt_modified")

    @property
    @pulumi.getter(name="isApplied")
    def is_applied(self) -> int:
        """
        Whether the parameter changing is applied.
        """
        return pulumi.get(self, "is_applied")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        Changed parameter name.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="newValue")
    def new_value(self) -> str:
        """
        The new value of parameter.
        """
        return pulumi.get(self, "new_value")

    @property
    @pulumi.getter(name="oldValue")
    def old_value(self) -> str:
        """
        The old value of parameter.
        """
        return pulumi.get(self, "old_value")


@pulumi.output_type
class GetDbInstancesInstanceResult(dict):
    def __init__(__self__, *,
                 cache_size_postpaid: int,
                 cache_size_prepaid: int,
                 cluster_count_postpaid: int,
                 cluster_count_prepaid: int,
                 cpu_postpaid: int,
                 cpu_prepaid: int,
                 db_instance_description: str,
                 db_instance_id: str,
                 engine: str,
                 engine_minor_version: str,
                 engine_version: str,
                 gmt_created: str,
                 gmt_expired: str,
                 gmt_modified: str,
                 id: str,
                 lock_mode: str,
                 lock_reason: str,
                 memory_postpaid: int,
                 memory_prepaid: int,
                 payment_type: str,
                 region_id: str,
                 status: str,
                 sub_domain: str,
                 vpc_id: str,
                 vswitch_id: str,
                 zone_id: str):
        """
        :param int cache_size_postpaid: The sum of cache size for every `PayAsYouGo` clusters in DBInstance.
        :param int cache_size_prepaid: The sum of cache size for every `Subscription` clusters in DBInstance.
        :param int cluster_count_postpaid: The sum of cluster counts for `PayAsYouGo` clusters in DBInstance.
        :param int cluster_count_prepaid: The sum of cluster counts for `Subscription` clusters in DBInstance.
        :param int cpu_postpaid: The sum of cpu resource amount for every `PayAsYouGo` clusters in DBInstance.
        :param int cpu_prepaid: The sum of cpu resource amount for every `Subscription` clusters in DBInstance.
        :param str db_instance_description: The DBInstance description.
        :param str db_instance_id: The instance ID.
        :param str engine: The Engine of the DBInstance.
        :param str engine_minor_version: The engine minor version of the DBInstance.
        :param str engine_version: The engine version of the DBInstance.
        :param str gmt_created: The time when DBInstance is created.
        :param str gmt_expired: The time when DBInstance will be expired. Available on `Subscription` DBInstance.
        :param str gmt_modified: The time when DBInstance is modified.
        :param str lock_mode: The lock mode of the instance. Set the value to lock, which specifies that the instance is locked when it automatically expires or has an overdue payment.
        :param str lock_reason: The reason why the instance is locked.
        :param int memory_postpaid: The sum of memory resource amount offor every `PayAsYouGo` clusters in DBInstance.
        :param int memory_prepaid: The sum of memory resource amount offor every `Subscription` clusters in DBInstance.
        :param str payment_type: The payment type of the resource. Valid values: `PayAsYouGo`,`Subscription`.
        :param str region_id: The ID of region for DBInstance.
        :param str status: The status of the DBInstance. Valid values: `ACTIVATION`,`CREATING`,`DELETING`,`RESTARTING`,`ORDER_PREPARING`.
        :param str sub_domain: The sub domain of DBInstance.
        :param str vpc_id: The ID of the VPC for DBInstance.
        :param str vswitch_id: The ID of vswitch for DBInstance.
        :param str zone_id: The ID of zone for DBInstance.
        """
        pulumi.set(__self__, "cache_size_postpaid", cache_size_postpaid)
        pulumi.set(__self__, "cache_size_prepaid", cache_size_prepaid)
        pulumi.set(__self__, "cluster_count_postpaid", cluster_count_postpaid)
        pulumi.set(__self__, "cluster_count_prepaid", cluster_count_prepaid)
        pulumi.set(__self__, "cpu_postpaid", cpu_postpaid)
        pulumi.set(__self__, "cpu_prepaid", cpu_prepaid)
        pulumi.set(__self__, "db_instance_description", db_instance_description)
        pulumi.set(__self__, "db_instance_id", db_instance_id)
        pulumi.set(__self__, "engine", engine)
        pulumi.set(__self__, "engine_minor_version", engine_minor_version)
        pulumi.set(__self__, "engine_version", engine_version)
        pulumi.set(__self__, "gmt_created", gmt_created)
        pulumi.set(__self__, "gmt_expired", gmt_expired)
        pulumi.set(__self__, "gmt_modified", gmt_modified)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "lock_mode", lock_mode)
        pulumi.set(__self__, "lock_reason", lock_reason)
        pulumi.set(__self__, "memory_postpaid", memory_postpaid)
        pulumi.set(__self__, "memory_prepaid", memory_prepaid)
        pulumi.set(__self__, "payment_type", payment_type)
        pulumi.set(__self__, "region_id", region_id)
        pulumi.set(__self__, "status", status)
        pulumi.set(__self__, "sub_domain", sub_domain)
        pulumi.set(__self__, "vpc_id", vpc_id)
        pulumi.set(__self__, "vswitch_id", vswitch_id)
        pulumi.set(__self__, "zone_id", zone_id)

    @property
    @pulumi.getter(name="cacheSizePostpaid")
    def cache_size_postpaid(self) -> int:
        """
        The sum of cache size for every `PayAsYouGo` clusters in DBInstance.
        """
        return pulumi.get(self, "cache_size_postpaid")

    @property
    @pulumi.getter(name="cacheSizePrepaid")
    def cache_size_prepaid(self) -> int:
        """
        The sum of cache size for every `Subscription` clusters in DBInstance.
        """
        return pulumi.get(self, "cache_size_prepaid")

    @property
    @pulumi.getter(name="clusterCountPostpaid")
    def cluster_count_postpaid(self) -> int:
        """
        The sum of cluster counts for `PayAsYouGo` clusters in DBInstance.
        """
        return pulumi.get(self, "cluster_count_postpaid")

    @property
    @pulumi.getter(name="clusterCountPrepaid")
    def cluster_count_prepaid(self) -> int:
        """
        The sum of cluster counts for `Subscription` clusters in DBInstance.
        """
        return pulumi.get(self, "cluster_count_prepaid")

    @property
    @pulumi.getter(name="cpuPostpaid")
    def cpu_postpaid(self) -> int:
        """
        The sum of cpu resource amount for every `PayAsYouGo` clusters in DBInstance.
        """
        return pulumi.get(self, "cpu_postpaid")

    @property
    @pulumi.getter(name="cpuPrepaid")
    def cpu_prepaid(self) -> int:
        """
        The sum of cpu resource amount for every `Subscription` clusters in DBInstance.
        """
        return pulumi.get(self, "cpu_prepaid")

    @property
    @pulumi.getter(name="dbInstanceDescription")
    def db_instance_description(self) -> str:
        """
        The DBInstance description.
        """
        return pulumi.get(self, "db_instance_description")

    @property
    @pulumi.getter(name="dbInstanceId")
    def db_instance_id(self) -> str:
        """
        The instance ID.
        """
        return pulumi.get(self, "db_instance_id")

    @property
    @pulumi.getter
    def engine(self) -> str:
        """
        The Engine of the DBInstance.
        """
        return pulumi.get(self, "engine")

    @property
    @pulumi.getter(name="engineMinorVersion")
    def engine_minor_version(self) -> str:
        """
        The engine minor version of the DBInstance.
        """
        return pulumi.get(self, "engine_minor_version")

    @property
    @pulumi.getter(name="engineVersion")
    def engine_version(self) -> str:
        """
        The engine version of the DBInstance.
        """
        return pulumi.get(self, "engine_version")

    @property
    @pulumi.getter(name="gmtCreated")
    def gmt_created(self) -> str:
        """
        The time when DBInstance is created.
        """
        return pulumi.get(self, "gmt_created")

    @property
    @pulumi.getter(name="gmtExpired")
    def gmt_expired(self) -> str:
        """
        The time when DBInstance will be expired. Available on `Subscription` DBInstance.
        """
        return pulumi.get(self, "gmt_expired")

    @property
    @pulumi.getter(name="gmtModified")
    def gmt_modified(self) -> str:
        """
        The time when DBInstance is modified.
        """
        return pulumi.get(self, "gmt_modified")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="lockMode")
    def lock_mode(self) -> str:
        """
        The lock mode of the instance. Set the value to lock, which specifies that the instance is locked when it automatically expires or has an overdue payment.
        """
        return pulumi.get(self, "lock_mode")

    @property
    @pulumi.getter(name="lockReason")
    def lock_reason(self) -> str:
        """
        The reason why the instance is locked.
        """
        return pulumi.get(self, "lock_reason")

    @property
    @pulumi.getter(name="memoryPostpaid")
    def memory_postpaid(self) -> int:
        """
        The sum of memory resource amount offor every `PayAsYouGo` clusters in DBInstance.
        """
        return pulumi.get(self, "memory_postpaid")

    @property
    @pulumi.getter(name="memoryPrepaid")
    def memory_prepaid(self) -> int:
        """
        The sum of memory resource amount offor every `Subscription` clusters in DBInstance.
        """
        return pulumi.get(self, "memory_prepaid")

    @property
    @pulumi.getter(name="paymentType")
    def payment_type(self) -> str:
        """
        The payment type of the resource. Valid values: `PayAsYouGo`,`Subscription`.
        """
        return pulumi.get(self, "payment_type")

    @property
    @pulumi.getter(name="regionId")
    def region_id(self) -> str:
        """
        The ID of region for DBInstance.
        """
        return pulumi.get(self, "region_id")

    @property
    @pulumi.getter
    def status(self) -> str:
        """
        The status of the DBInstance. Valid values: `ACTIVATION`,`CREATING`,`DELETING`,`RESTARTING`,`ORDER_PREPARING`.
        """
        return pulumi.get(self, "status")

    @property
    @pulumi.getter(name="subDomain")
    def sub_domain(self) -> str:
        """
        The sub domain of DBInstance.
        """
        return pulumi.get(self, "sub_domain")

    @property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> str:
        """
        The ID of the VPC for DBInstance.
        """
        return pulumi.get(self, "vpc_id")

    @property
    @pulumi.getter(name="vswitchId")
    def vswitch_id(self) -> str:
        """
        The ID of vswitch for DBInstance.
        """
        return pulumi.get(self, "vswitch_id")

    @property
    @pulumi.getter(name="zoneId")
    def zone_id(self) -> str:
        """
        The ID of zone for DBInstance.
        """
        return pulumi.get(self, "zone_id")





# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['ProjectArgs', 'Project']

@pulumi.input_type
class ProjectArgs:
    def __init__(__self__, *,
                 comment: Optional[pulumi.Input[str]] = None,
                 default_quota: Optional[pulumi.Input[str]] = None,
                 ip_white_list: Optional[pulumi.Input['ProjectIpWhiteListArgs']] = None,
                 is_logical: Optional[pulumi.Input[str]] = None,
                 product_type: Optional[pulumi.Input[str]] = None,
                 project_name: Optional[pulumi.Input[str]] = None,
                 properties: Optional[pulumi.Input['ProjectPropertiesArgs']] = None,
                 security_properties: Optional[pulumi.Input['ProjectSecurityPropertiesArgs']] = None,
                 status: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None):
        """
        The set of arguments for constructing a Project resource.
        :param pulumi.Input[str] comment: Project description information. The length is 1 to 256 English or Chinese characters. The default value is blank.
        :param pulumi.Input[str] default_quota: Used to implement computing resource allocation. If the calculation Quota is not specified, the default Quota resource will be consumed by jobs initiated by the project. For more information about computing resource usage, see [Computing Resource Usage](https://www.alibabacloud.com/help/en/maxcompute/user-guide/use-of-computing-resources).
        :param pulumi.Input['ProjectIpWhiteListArgs'] ip_white_list: IP whitelist See `ip_white_list` below.
        :param pulumi.Input[str] is_logical: Logical deletion, value: (true/false) true: In this case, the project status will be changed to 'DELETING' and completely deleted after 14 days. false: immediately deleted, that is, completely deleted, permanently unrecoverable.
        :param pulumi.Input[str] product_type: Quota payment type, support `PayAsYouGo`, `Subscription`, `Dev`.
        :param pulumi.Input[str] project_name: The name begins with a letter, containing letters, digits, and underscores (_). It can be 3 to 28 characters in length and is globally unique.
        :param pulumi.Input['ProjectPropertiesArgs'] properties: Project base attributes See `properties` below.
        :param pulumi.Input['ProjectSecurityPropertiesArgs'] security_properties: Security-related attributes See `security_properties` below.
        :param pulumi.Input[str] status: The project status. Default value: AVAILABLE. Value: (AVAILABLE/READONLY/FROZEN/DELETING)
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tag of the resource
        """
        if comment is not None:
            pulumi.set(__self__, "comment", comment)
        if default_quota is not None:
            pulumi.set(__self__, "default_quota", default_quota)
        if ip_white_list is not None:
            pulumi.set(__self__, "ip_white_list", ip_white_list)
        if is_logical is not None:
            pulumi.set(__self__, "is_logical", is_logical)
        if product_type is not None:
            pulumi.set(__self__, "product_type", product_type)
        if project_name is not None:
            pulumi.set(__self__, "project_name", project_name)
        if properties is not None:
            pulumi.set(__self__, "properties", properties)
        if security_properties is not None:
            pulumi.set(__self__, "security_properties", security_properties)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @property
    @pulumi.getter
    def comment(self) -> Optional[pulumi.Input[str]]:
        """
        Project description information. The length is 1 to 256 English or Chinese characters. The default value is blank.
        """
        return pulumi.get(self, "comment")

    @comment.setter
    def comment(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "comment", value)

    @property
    @pulumi.getter(name="defaultQuota")
    def default_quota(self) -> Optional[pulumi.Input[str]]:
        """
        Used to implement computing resource allocation. If the calculation Quota is not specified, the default Quota resource will be consumed by jobs initiated by the project. For more information about computing resource usage, see [Computing Resource Usage](https://www.alibabacloud.com/help/en/maxcompute/user-guide/use-of-computing-resources).
        """
        return pulumi.get(self, "default_quota")

    @default_quota.setter
    def default_quota(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "default_quota", value)

    @property
    @pulumi.getter(name="ipWhiteList")
    def ip_white_list(self) -> Optional[pulumi.Input['ProjectIpWhiteListArgs']]:
        """
        IP whitelist See `ip_white_list` below.
        """
        return pulumi.get(self, "ip_white_list")

    @ip_white_list.setter
    def ip_white_list(self, value: Optional[pulumi.Input['ProjectIpWhiteListArgs']]):
        pulumi.set(self, "ip_white_list", value)

    @property
    @pulumi.getter(name="isLogical")
    def is_logical(self) -> Optional[pulumi.Input[str]]:
        """
        Logical deletion, value: (true/false) true: In this case, the project status will be changed to 'DELETING' and completely deleted after 14 days. false: immediately deleted, that is, completely deleted, permanently unrecoverable.
        """
        return pulumi.get(self, "is_logical")

    @is_logical.setter
    def is_logical(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "is_logical", value)

    @property
    @pulumi.getter(name="productType")
    def product_type(self) -> Optional[pulumi.Input[str]]:
        """
        Quota payment type, support `PayAsYouGo`, `Subscription`, `Dev`.
        """
        return pulumi.get(self, "product_type")

    @product_type.setter
    def product_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "product_type", value)

    @property
    @pulumi.getter(name="projectName")
    def project_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name begins with a letter, containing letters, digits, and underscores (_). It can be 3 to 28 characters in length and is globally unique.
        """
        return pulumi.get(self, "project_name")

    @project_name.setter
    def project_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "project_name", value)

    @property
    @pulumi.getter
    def properties(self) -> Optional[pulumi.Input['ProjectPropertiesArgs']]:
        """
        Project base attributes See `properties` below.
        """
        return pulumi.get(self, "properties")

    @properties.setter
    def properties(self, value: Optional[pulumi.Input['ProjectPropertiesArgs']]):
        pulumi.set(self, "properties", value)

    @property
    @pulumi.getter(name="securityProperties")
    def security_properties(self) -> Optional[pulumi.Input['ProjectSecurityPropertiesArgs']]:
        """
        Security-related attributes See `security_properties` below.
        """
        return pulumi.get(self, "security_properties")

    @security_properties.setter
    def security_properties(self, value: Optional[pulumi.Input['ProjectSecurityPropertiesArgs']]):
        pulumi.set(self, "security_properties", value)

    @property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[str]]:
        """
        The project status. Default value: AVAILABLE. Value: (AVAILABLE/READONLY/FROZEN/DELETING)
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "status", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        The tag of the resource
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)


@pulumi.input_type
class _ProjectState:
    def __init__(__self__, *,
                 comment: Optional[pulumi.Input[str]] = None,
                 create_time: Optional[pulumi.Input[str]] = None,
                 default_quota: Optional[pulumi.Input[str]] = None,
                 ip_white_list: Optional[pulumi.Input['ProjectIpWhiteListArgs']] = None,
                 is_logical: Optional[pulumi.Input[str]] = None,
                 owner: Optional[pulumi.Input[str]] = None,
                 product_type: Optional[pulumi.Input[str]] = None,
                 project_name: Optional[pulumi.Input[str]] = None,
                 properties: Optional[pulumi.Input['ProjectPropertiesArgs']] = None,
                 security_properties: Optional[pulumi.Input['ProjectSecurityPropertiesArgs']] = None,
                 status: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 type: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Project resources.
        :param pulumi.Input[str] comment: Project description information. The length is 1 to 256 English or Chinese characters. The default value is blank.
        :param pulumi.Input[str] create_time: Represents the creation time of the project
        :param pulumi.Input[str] default_quota: Used to implement computing resource allocation. If the calculation Quota is not specified, the default Quota resource will be consumed by jobs initiated by the project. For more information about computing resource usage, see [Computing Resource Usage](https://www.alibabacloud.com/help/en/maxcompute/user-guide/use-of-computing-resources).
        :param pulumi.Input['ProjectIpWhiteListArgs'] ip_white_list: IP whitelist See `ip_white_list` below.
        :param pulumi.Input[str] is_logical: Logical deletion, value: (true/false) true: In this case, the project status will be changed to 'DELETING' and completely deleted after 14 days. false: immediately deleted, that is, completely deleted, permanently unrecoverable.
        :param pulumi.Input[str] owner: Project owner
        :param pulumi.Input[str] product_type: Quota payment type, support `PayAsYouGo`, `Subscription`, `Dev`.
        :param pulumi.Input[str] project_name: The name begins with a letter, containing letters, digits, and underscores (_). It can be 3 to 28 characters in length and is globally unique.
        :param pulumi.Input['ProjectPropertiesArgs'] properties: Project base attributes See `properties` below.
        :param pulumi.Input['ProjectSecurityPropertiesArgs'] security_properties: Security-related attributes See `security_properties` below.
        :param pulumi.Input[str] status: The project status. Default value: AVAILABLE. Value: (AVAILABLE/READONLY/FROZEN/DELETING)
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tag of the resource
        :param pulumi.Input[str] type: Project type
        """
        if comment is not None:
            pulumi.set(__self__, "comment", comment)
        if create_time is not None:
            pulumi.set(__self__, "create_time", create_time)
        if default_quota is not None:
            pulumi.set(__self__, "default_quota", default_quota)
        if ip_white_list is not None:
            pulumi.set(__self__, "ip_white_list", ip_white_list)
        if is_logical is not None:
            pulumi.set(__self__, "is_logical", is_logical)
        if owner is not None:
            pulumi.set(__self__, "owner", owner)
        if product_type is not None:
            pulumi.set(__self__, "product_type", product_type)
        if project_name is not None:
            pulumi.set(__self__, "project_name", project_name)
        if properties is not None:
            pulumi.set(__self__, "properties", properties)
        if security_properties is not None:
            pulumi.set(__self__, "security_properties", security_properties)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if type is not None:
            pulumi.set(__self__, "type", type)

    @property
    @pulumi.getter
    def comment(self) -> Optional[pulumi.Input[str]]:
        """
        Project description information. The length is 1 to 256 English or Chinese characters. The default value is blank.
        """
        return pulumi.get(self, "comment")

    @comment.setter
    def comment(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "comment", value)

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> Optional[pulumi.Input[str]]:
        """
        Represents the creation time of the project
        """
        return pulumi.get(self, "create_time")

    @create_time.setter
    def create_time(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "create_time", value)

    @property
    @pulumi.getter(name="defaultQuota")
    def default_quota(self) -> Optional[pulumi.Input[str]]:
        """
        Used to implement computing resource allocation. If the calculation Quota is not specified, the default Quota resource will be consumed by jobs initiated by the project. For more information about computing resource usage, see [Computing Resource Usage](https://www.alibabacloud.com/help/en/maxcompute/user-guide/use-of-computing-resources).
        """
        return pulumi.get(self, "default_quota")

    @default_quota.setter
    def default_quota(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "default_quota", value)

    @property
    @pulumi.getter(name="ipWhiteList")
    def ip_white_list(self) -> Optional[pulumi.Input['ProjectIpWhiteListArgs']]:
        """
        IP whitelist See `ip_white_list` below.
        """
        return pulumi.get(self, "ip_white_list")

    @ip_white_list.setter
    def ip_white_list(self, value: Optional[pulumi.Input['ProjectIpWhiteListArgs']]):
        pulumi.set(self, "ip_white_list", value)

    @property
    @pulumi.getter(name="isLogical")
    def is_logical(self) -> Optional[pulumi.Input[str]]:
        """
        Logical deletion, value: (true/false) true: In this case, the project status will be changed to 'DELETING' and completely deleted after 14 days. false: immediately deleted, that is, completely deleted, permanently unrecoverable.
        """
        return pulumi.get(self, "is_logical")

    @is_logical.setter
    def is_logical(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "is_logical", value)

    @property
    @pulumi.getter
    def owner(self) -> Optional[pulumi.Input[str]]:
        """
        Project owner
        """
        return pulumi.get(self, "owner")

    @owner.setter
    def owner(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "owner", value)

    @property
    @pulumi.getter(name="productType")
    def product_type(self) -> Optional[pulumi.Input[str]]:
        """
        Quota payment type, support `PayAsYouGo`, `Subscription`, `Dev`.
        """
        return pulumi.get(self, "product_type")

    @product_type.setter
    def product_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "product_type", value)

    @property
    @pulumi.getter(name="projectName")
    def project_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name begins with a letter, containing letters, digits, and underscores (_). It can be 3 to 28 characters in length and is globally unique.
        """
        return pulumi.get(self, "project_name")

    @project_name.setter
    def project_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "project_name", value)

    @property
    @pulumi.getter
    def properties(self) -> Optional[pulumi.Input['ProjectPropertiesArgs']]:
        """
        Project base attributes See `properties` below.
        """
        return pulumi.get(self, "properties")

    @properties.setter
    def properties(self, value: Optional[pulumi.Input['ProjectPropertiesArgs']]):
        pulumi.set(self, "properties", value)

    @property
    @pulumi.getter(name="securityProperties")
    def security_properties(self) -> Optional[pulumi.Input['ProjectSecurityPropertiesArgs']]:
        """
        Security-related attributes See `security_properties` below.
        """
        return pulumi.get(self, "security_properties")

    @security_properties.setter
    def security_properties(self, value: Optional[pulumi.Input['ProjectSecurityPropertiesArgs']]):
        pulumi.set(self, "security_properties", value)

    @property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[str]]:
        """
        The project status. Default value: AVAILABLE. Value: (AVAILABLE/READONLY/FROZEN/DELETING)
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "status", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        The tag of the resource
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)

    @property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[str]]:
        """
        Project type
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "type", value)


class Project(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 comment: Optional[pulumi.Input[str]] = None,
                 default_quota: Optional[pulumi.Input[str]] = None,
                 ip_white_list: Optional[pulumi.Input[Union['ProjectIpWhiteListArgs', 'ProjectIpWhiteListArgsDict']]] = None,
                 is_logical: Optional[pulumi.Input[str]] = None,
                 product_type: Optional[pulumi.Input[str]] = None,
                 project_name: Optional[pulumi.Input[str]] = None,
                 properties: Optional[pulumi.Input[Union['ProjectPropertiesArgs', 'ProjectPropertiesArgsDict']]] = None,
                 security_properties: Optional[pulumi.Input[Union['ProjectSecurityPropertiesArgs', 'ProjectSecurityPropertiesArgsDict']]] = None,
                 status: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 __props__=None):
        """
        Provides a Max Compute Project resource.

        MaxCompute project .

        For information about Max Compute Project and how to use it, see [What is Project](https://www.alibabacloud.com/help/en/maxcompute/).

        > **NOTE:** Available since v1.77.0.

        > **NOTE:** Field `name`, `specification_type`, `order_type` has been removed from provider version 1.227.1.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "tf_example"
        default = alicloud.maxcompute.Project("default",
            default_quota="默认后付费Quota",
            project_name=name,
            comment=name,
            product_type="PayAsYouGo")
        ```

        ## Import

        Max Compute Project can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:maxcompute/project:Project example <id>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] comment: Project description information. The length is 1 to 256 English or Chinese characters. The default value is blank.
        :param pulumi.Input[str] default_quota: Used to implement computing resource allocation. If the calculation Quota is not specified, the default Quota resource will be consumed by jobs initiated by the project. For more information about computing resource usage, see [Computing Resource Usage](https://www.alibabacloud.com/help/en/maxcompute/user-guide/use-of-computing-resources).
        :param pulumi.Input[Union['ProjectIpWhiteListArgs', 'ProjectIpWhiteListArgsDict']] ip_white_list: IP whitelist See `ip_white_list` below.
        :param pulumi.Input[str] is_logical: Logical deletion, value: (true/false) true: In this case, the project status will be changed to 'DELETING' and completely deleted after 14 days. false: immediately deleted, that is, completely deleted, permanently unrecoverable.
        :param pulumi.Input[str] product_type: Quota payment type, support `PayAsYouGo`, `Subscription`, `Dev`.
        :param pulumi.Input[str] project_name: The name begins with a letter, containing letters, digits, and underscores (_). It can be 3 to 28 characters in length and is globally unique.
        :param pulumi.Input[Union['ProjectPropertiesArgs', 'ProjectPropertiesArgsDict']] properties: Project base attributes See `properties` below.
        :param pulumi.Input[Union['ProjectSecurityPropertiesArgs', 'ProjectSecurityPropertiesArgsDict']] security_properties: Security-related attributes See `security_properties` below.
        :param pulumi.Input[str] status: The project status. Default value: AVAILABLE. Value: (AVAILABLE/READONLY/FROZEN/DELETING)
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tag of the resource
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[ProjectArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Max Compute Project resource.

        MaxCompute project .

        For information about Max Compute Project and how to use it, see [What is Project](https://www.alibabacloud.com/help/en/maxcompute/).

        > **NOTE:** Available since v1.77.0.

        > **NOTE:** Field `name`, `specification_type`, `order_type` has been removed from provider version 1.227.1.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "tf_example"
        default = alicloud.maxcompute.Project("default",
            default_quota="默认后付费Quota",
            project_name=name,
            comment=name,
            product_type="PayAsYouGo")
        ```

        ## Import

        Max Compute Project can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:maxcompute/project:Project example <id>
        ```

        :param str resource_name: The name of the resource.
        :param ProjectArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ProjectArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 comment: Optional[pulumi.Input[str]] = None,
                 default_quota: Optional[pulumi.Input[str]] = None,
                 ip_white_list: Optional[pulumi.Input[Union['ProjectIpWhiteListArgs', 'ProjectIpWhiteListArgsDict']]] = None,
                 is_logical: Optional[pulumi.Input[str]] = None,
                 product_type: Optional[pulumi.Input[str]] = None,
                 project_name: Optional[pulumi.Input[str]] = None,
                 properties: Optional[pulumi.Input[Union['ProjectPropertiesArgs', 'ProjectPropertiesArgsDict']]] = None,
                 security_properties: Optional[pulumi.Input[Union['ProjectSecurityPropertiesArgs', 'ProjectSecurityPropertiesArgsDict']]] = None,
                 status: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ProjectArgs.__new__(ProjectArgs)

            __props__.__dict__["comment"] = comment
            __props__.__dict__["default_quota"] = default_quota
            __props__.__dict__["ip_white_list"] = ip_white_list
            __props__.__dict__["is_logical"] = is_logical
            __props__.__dict__["product_type"] = product_type
            __props__.__dict__["project_name"] = project_name
            __props__.__dict__["properties"] = properties
            __props__.__dict__["security_properties"] = security_properties
            __props__.__dict__["status"] = status
            __props__.__dict__["tags"] = tags
            __props__.__dict__["create_time"] = None
            __props__.__dict__["owner"] = None
            __props__.__dict__["type"] = None
        super(Project, __self__).__init__(
            'alicloud:maxcompute/project:Project',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            comment: Optional[pulumi.Input[str]] = None,
            create_time: Optional[pulumi.Input[str]] = None,
            default_quota: Optional[pulumi.Input[str]] = None,
            ip_white_list: Optional[pulumi.Input[Union['ProjectIpWhiteListArgs', 'ProjectIpWhiteListArgsDict']]] = None,
            is_logical: Optional[pulumi.Input[str]] = None,
            owner: Optional[pulumi.Input[str]] = None,
            product_type: Optional[pulumi.Input[str]] = None,
            project_name: Optional[pulumi.Input[str]] = None,
            properties: Optional[pulumi.Input[Union['ProjectPropertiesArgs', 'ProjectPropertiesArgsDict']]] = None,
            security_properties: Optional[pulumi.Input[Union['ProjectSecurityPropertiesArgs', 'ProjectSecurityPropertiesArgsDict']]] = None,
            status: Optional[pulumi.Input[str]] = None,
            tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            type: Optional[pulumi.Input[str]] = None) -> 'Project':
        """
        Get an existing Project resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] comment: Project description information. The length is 1 to 256 English or Chinese characters. The default value is blank.
        :param pulumi.Input[str] create_time: Represents the creation time of the project
        :param pulumi.Input[str] default_quota: Used to implement computing resource allocation. If the calculation Quota is not specified, the default Quota resource will be consumed by jobs initiated by the project. For more information about computing resource usage, see [Computing Resource Usage](https://www.alibabacloud.com/help/en/maxcompute/user-guide/use-of-computing-resources).
        :param pulumi.Input[Union['ProjectIpWhiteListArgs', 'ProjectIpWhiteListArgsDict']] ip_white_list: IP whitelist See `ip_white_list` below.
        :param pulumi.Input[str] is_logical: Logical deletion, value: (true/false) true: In this case, the project status will be changed to 'DELETING' and completely deleted after 14 days. false: immediately deleted, that is, completely deleted, permanently unrecoverable.
        :param pulumi.Input[str] owner: Project owner
        :param pulumi.Input[str] product_type: Quota payment type, support `PayAsYouGo`, `Subscription`, `Dev`.
        :param pulumi.Input[str] project_name: The name begins with a letter, containing letters, digits, and underscores (_). It can be 3 to 28 characters in length and is globally unique.
        :param pulumi.Input[Union['ProjectPropertiesArgs', 'ProjectPropertiesArgsDict']] properties: Project base attributes See `properties` below.
        :param pulumi.Input[Union['ProjectSecurityPropertiesArgs', 'ProjectSecurityPropertiesArgsDict']] security_properties: Security-related attributes See `security_properties` below.
        :param pulumi.Input[str] status: The project status. Default value: AVAILABLE. Value: (AVAILABLE/READONLY/FROZEN/DELETING)
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tag of the resource
        :param pulumi.Input[str] type: Project type
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ProjectState.__new__(_ProjectState)

        __props__.__dict__["comment"] = comment
        __props__.__dict__["create_time"] = create_time
        __props__.__dict__["default_quota"] = default_quota
        __props__.__dict__["ip_white_list"] = ip_white_list
        __props__.__dict__["is_logical"] = is_logical
        __props__.__dict__["owner"] = owner
        __props__.__dict__["product_type"] = product_type
        __props__.__dict__["project_name"] = project_name
        __props__.__dict__["properties"] = properties
        __props__.__dict__["security_properties"] = security_properties
        __props__.__dict__["status"] = status
        __props__.__dict__["tags"] = tags
        __props__.__dict__["type"] = type
        return Project(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def comment(self) -> pulumi.Output[Optional[str]]:
        """
        Project description information. The length is 1 to 256 English or Chinese characters. The default value is blank.
        """
        return pulumi.get(self, "comment")

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> pulumi.Output[str]:
        """
        Represents the creation time of the project
        """
        return pulumi.get(self, "create_time")

    @property
    @pulumi.getter(name="defaultQuota")
    def default_quota(self) -> pulumi.Output[Optional[str]]:
        """
        Used to implement computing resource allocation. If the calculation Quota is not specified, the default Quota resource will be consumed by jobs initiated by the project. For more information about computing resource usage, see [Computing Resource Usage](https://www.alibabacloud.com/help/en/maxcompute/user-guide/use-of-computing-resources).
        """
        return pulumi.get(self, "default_quota")

    @property
    @pulumi.getter(name="ipWhiteList")
    def ip_white_list(self) -> pulumi.Output[Optional['outputs.ProjectIpWhiteList']]:
        """
        IP whitelist See `ip_white_list` below.
        """
        return pulumi.get(self, "ip_white_list")

    @property
    @pulumi.getter(name="isLogical")
    def is_logical(self) -> pulumi.Output[Optional[str]]:
        """
        Logical deletion, value: (true/false) true: In this case, the project status will be changed to 'DELETING' and completely deleted after 14 days. false: immediately deleted, that is, completely deleted, permanently unrecoverable.
        """
        return pulumi.get(self, "is_logical")

    @property
    @pulumi.getter
    def owner(self) -> pulumi.Output[str]:
        """
        Project owner
        """
        return pulumi.get(self, "owner")

    @property
    @pulumi.getter(name="productType")
    def product_type(self) -> pulumi.Output[Optional[str]]:
        """
        Quota payment type, support `PayAsYouGo`, `Subscription`, `Dev`.
        """
        return pulumi.get(self, "product_type")

    @property
    @pulumi.getter(name="projectName")
    def project_name(self) -> pulumi.Output[str]:
        """
        The name begins with a letter, containing letters, digits, and underscores (_). It can be 3 to 28 characters in length and is globally unique.
        """
        return pulumi.get(self, "project_name")

    @property
    @pulumi.getter
    def properties(self) -> pulumi.Output['outputs.ProjectProperties']:
        """
        Project base attributes See `properties` below.
        """
        return pulumi.get(self, "properties")

    @property
    @pulumi.getter(name="securityProperties")
    def security_properties(self) -> pulumi.Output['outputs.ProjectSecurityProperties']:
        """
        Security-related attributes See `security_properties` below.
        """
        return pulumi.get(self, "security_properties")

    @property
    @pulumi.getter
    def status(self) -> pulumi.Output[str]:
        """
        The project status. Default value: AVAILABLE. Value: (AVAILABLE/READONLY/FROZEN/DELETING)
        """
        return pulumi.get(self, "status")

    @property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        The tag of the resource
        """
        return pulumi.get(self, "tags")

    @property
    @pulumi.getter
    def type(self) -> pulumi.Output[str]:
        """
        Project type
        """
        return pulumi.get(self, "type")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['Ipv4GatewayArgs', 'Ipv4Gateway']

@pulumi.input_type
class Ipv4GatewayArgs:
    def __init__(__self__, *,
                 vpc_id: pulumi.Input[str],
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 ipv4_gateway_description: Optional[pulumi.Input[str]] = None,
                 ipv4_gateway_name: Optional[pulumi.Input[str]] = None,
                 resource_group_id: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None):
        """
        The set of arguments for constructing a Ipv4Gateway resource.
        :param pulumi.Input[str] vpc_id: The ID of the virtual private cloud (VPC) where you want to create the IPv4 gateway. You can create only one IPv4 gateway in a VPC.
        :param pulumi.Input[bool] dry_run: Whether to PreCheck only this request. Value:-**true**: The check request is sent without creating an IPv4 Gateway. Check items include whether required parameters, request format, and business restrictions are filled in. If the check does not pass, the corresponding error is returned. If the check passes, the error code 'DryRunOperation' is returned '.-**false** (default): Sends a normal request, returns an HTTP 2xx status code and directly creates an IPv4 Gateway.
        :param pulumi.Input[bool] enabled: Whether the IPv4 gateway is active or not. Valid values are **true** and **false**.
        :param pulumi.Input[str] ipv4_gateway_description: The description of the IPv4 gateway. The description must be 2 to 256 characters in length. It must start with a letter but cannot start with http:// or https://.
        :param pulumi.Input[str] ipv4_gateway_name: The name of the IPv4 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, periods (.), underscores (_), and hyphens (-). It must start with a letter.
        :param pulumi.Input[str] resource_group_id: The ID of the resource group to which the instance belongs.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tags of the current resource.
        """
        pulumi.set(__self__, "vpc_id", vpc_id)
        if dry_run is not None:
            pulumi.set(__self__, "dry_run", dry_run)
        if enabled is not None:
            pulumi.set(__self__, "enabled", enabled)
        if ipv4_gateway_description is not None:
            pulumi.set(__self__, "ipv4_gateway_description", ipv4_gateway_description)
        if ipv4_gateway_name is not None:
            pulumi.set(__self__, "ipv4_gateway_name", ipv4_gateway_name)
        if resource_group_id is not None:
            pulumi.set(__self__, "resource_group_id", resource_group_id)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> pulumi.Input[str]:
        """
        The ID of the virtual private cloud (VPC) where you want to create the IPv4 gateway. You can create only one IPv4 gateway in a VPC.
        """
        return pulumi.get(self, "vpc_id")

    @vpc_id.setter
    def vpc_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "vpc_id", value)

    @property
    @pulumi.getter(name="dryRun")
    def dry_run(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to PreCheck only this request. Value:-**true**: The check request is sent without creating an IPv4 Gateway. Check items include whether required parameters, request format, and business restrictions are filled in. If the check does not pass, the corresponding error is returned. If the check passes, the error code 'DryRunOperation' is returned '.-**false** (default): Sends a normal request, returns an HTTP 2xx status code and directly creates an IPv4 Gateway.
        """
        return pulumi.get(self, "dry_run")

    @dry_run.setter
    def dry_run(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "dry_run", value)

    @property
    @pulumi.getter
    def enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether the IPv4 gateway is active or not. Valid values are **true** and **false**.
        """
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter(name="ipv4GatewayDescription")
    def ipv4_gateway_description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the IPv4 gateway. The description must be 2 to 256 characters in length. It must start with a letter but cannot start with http:// or https://.
        """
        return pulumi.get(self, "ipv4_gateway_description")

    @ipv4_gateway_description.setter
    def ipv4_gateway_description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ipv4_gateway_description", value)

    @property
    @pulumi.getter(name="ipv4GatewayName")
    def ipv4_gateway_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the IPv4 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, periods (.), underscores (_), and hyphens (-). It must start with a letter.
        """
        return pulumi.get(self, "ipv4_gateway_name")

    @ipv4_gateway_name.setter
    def ipv4_gateway_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ipv4_gateway_name", value)

    @property
    @pulumi.getter(name="resourceGroupId")
    def resource_group_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the resource group to which the instance belongs.
        """
        return pulumi.get(self, "resource_group_id")

    @resource_group_id.setter
    def resource_group_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "resource_group_id", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        The tags of the current resource.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)


@pulumi.input_type
class _Ipv4GatewayState:
    def __init__(__self__, *,
                 create_time: Optional[pulumi.Input[str]] = None,
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 ipv4_gateway_description: Optional[pulumi.Input[str]] = None,
                 ipv4_gateway_id: Optional[pulumi.Input[str]] = None,
                 ipv4_gateway_name: Optional[pulumi.Input[str]] = None,
                 ipv4_gateway_route_table_id: Optional[pulumi.Input[str]] = None,
                 resource_group_id: Optional[pulumi.Input[str]] = None,
                 status: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 vpc_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Ipv4Gateway resources.
        :param pulumi.Input[str] create_time: The creation time of the resource.
        :param pulumi.Input[bool] dry_run: Whether to PreCheck only this request. Value:-**true**: The check request is sent without creating an IPv4 Gateway. Check items include whether required parameters, request format, and business restrictions are filled in. If the check does not pass, the corresponding error is returned. If the check passes, the error code 'DryRunOperation' is returned '.-**false** (default): Sends a normal request, returns an HTTP 2xx status code and directly creates an IPv4 Gateway.
        :param pulumi.Input[bool] enabled: Whether the IPv4 gateway is active or not. Valid values are **true** and **false**.
        :param pulumi.Input[str] ipv4_gateway_description: The description of the IPv4 gateway. The description must be 2 to 256 characters in length. It must start with a letter but cannot start with http:// or https://.
        :param pulumi.Input[str] ipv4_gateway_id: Resource primary key field.
        :param pulumi.Input[str] ipv4_gateway_name: The name of the IPv4 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, periods (.), underscores (_), and hyphens (-). It must start with a letter.
        :param pulumi.Input[str] ipv4_gateway_route_table_id: ID of the route table associated with IPv4 Gateway.
        :param pulumi.Input[str] resource_group_id: The ID of the resource group to which the instance belongs.
        :param pulumi.Input[str] status: The status of the resource.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tags of the current resource.
        :param pulumi.Input[str] vpc_id: The ID of the virtual private cloud (VPC) where you want to create the IPv4 gateway. You can create only one IPv4 gateway in a VPC.
        """
        if create_time is not None:
            pulumi.set(__self__, "create_time", create_time)
        if dry_run is not None:
            pulumi.set(__self__, "dry_run", dry_run)
        if enabled is not None:
            pulumi.set(__self__, "enabled", enabled)
        if ipv4_gateway_description is not None:
            pulumi.set(__self__, "ipv4_gateway_description", ipv4_gateway_description)
        if ipv4_gateway_id is not None:
            pulumi.set(__self__, "ipv4_gateway_id", ipv4_gateway_id)
        if ipv4_gateway_name is not None:
            pulumi.set(__self__, "ipv4_gateway_name", ipv4_gateway_name)
        if ipv4_gateway_route_table_id is not None:
            pulumi.set(__self__, "ipv4_gateway_route_table_id", ipv4_gateway_route_table_id)
        if resource_group_id is not None:
            pulumi.set(__self__, "resource_group_id", resource_group_id)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if vpc_id is not None:
            pulumi.set(__self__, "vpc_id", vpc_id)

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> Optional[pulumi.Input[str]]:
        """
        The creation time of the resource.
        """
        return pulumi.get(self, "create_time")

    @create_time.setter
    def create_time(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "create_time", value)

    @property
    @pulumi.getter(name="dryRun")
    def dry_run(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to PreCheck only this request. Value:-**true**: The check request is sent without creating an IPv4 Gateway. Check items include whether required parameters, request format, and business restrictions are filled in. If the check does not pass, the corresponding error is returned. If the check passes, the error code 'DryRunOperation' is returned '.-**false** (default): Sends a normal request, returns an HTTP 2xx status code and directly creates an IPv4 Gateway.
        """
        return pulumi.get(self, "dry_run")

    @dry_run.setter
    def dry_run(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "dry_run", value)

    @property
    @pulumi.getter
    def enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether the IPv4 gateway is active or not. Valid values are **true** and **false**.
        """
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter(name="ipv4GatewayDescription")
    def ipv4_gateway_description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the IPv4 gateway. The description must be 2 to 256 characters in length. It must start with a letter but cannot start with http:// or https://.
        """
        return pulumi.get(self, "ipv4_gateway_description")

    @ipv4_gateway_description.setter
    def ipv4_gateway_description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ipv4_gateway_description", value)

    @property
    @pulumi.getter(name="ipv4GatewayId")
    def ipv4_gateway_id(self) -> Optional[pulumi.Input[str]]:
        """
        Resource primary key field.
        """
        return pulumi.get(self, "ipv4_gateway_id")

    @ipv4_gateway_id.setter
    def ipv4_gateway_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ipv4_gateway_id", value)

    @property
    @pulumi.getter(name="ipv4GatewayName")
    def ipv4_gateway_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the IPv4 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, periods (.), underscores (_), and hyphens (-). It must start with a letter.
        """
        return pulumi.get(self, "ipv4_gateway_name")

    @ipv4_gateway_name.setter
    def ipv4_gateway_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ipv4_gateway_name", value)

    @property
    @pulumi.getter(name="ipv4GatewayRouteTableId")
    def ipv4_gateway_route_table_id(self) -> Optional[pulumi.Input[str]]:
        """
        ID of the route table associated with IPv4 Gateway.
        """
        return pulumi.get(self, "ipv4_gateway_route_table_id")

    @ipv4_gateway_route_table_id.setter
    def ipv4_gateway_route_table_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ipv4_gateway_route_table_id", value)

    @property
    @pulumi.getter(name="resourceGroupId")
    def resource_group_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the resource group to which the instance belongs.
        """
        return pulumi.get(self, "resource_group_id")

    @resource_group_id.setter
    def resource_group_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "resource_group_id", value)

    @property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[str]]:
        """
        The status of the resource.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "status", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        The tags of the current resource.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)

    @property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the virtual private cloud (VPC) where you want to create the IPv4 gateway. You can create only one IPv4 gateway in a VPC.
        """
        return pulumi.get(self, "vpc_id")

    @vpc_id.setter
    def vpc_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "vpc_id", value)


class Ipv4Gateway(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 ipv4_gateway_description: Optional[pulumi.Input[str]] = None,
                 ipv4_gateway_name: Optional[pulumi.Input[str]] = None,
                 resource_group_id: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 vpc_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Provides a Vpc Ipv4 Gateway resource.

        For information about Vpc Ipv4 Gateway and how to use it, see [What is Ipv4 Gateway](https://www.alibabacloud.com/help/en/virtual-private-cloud/latest/createipv4gateway).

        > **NOTE:** Available in v1.181.0+.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "tf-testacc-example"
        default = alicloud.resourcemanager.ResourceGroup("default",
            display_name="tf-testAcc-rg665",
            resource_group_name=name)
        modify = alicloud.resourcemanager.ResourceGroup("modify",
            display_name="tf-testAcc-rg298",
            resource_group_name=f"{name}1")
        default_network = alicloud.vpc.Network("default",
            vpc_name=f"{name}2",
            cidr_block="10.0.0.0/8")
        default_ipv4_gateway = alicloud.vpc.Ipv4Gateway("default",
            ipv4_gateway_name=name,
            ipv4_gateway_description="tf-testAcc-Ipv4Gateway",
            resource_group_id=default.id,
            vpc_id=default_network.id)
        ```

        ## Import

        Vpc Ipv4 Gateway can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:vpc/ipv4Gateway:Ipv4Gateway example <id>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] dry_run: Whether to PreCheck only this request. Value:-**true**: The check request is sent without creating an IPv4 Gateway. Check items include whether required parameters, request format, and business restrictions are filled in. If the check does not pass, the corresponding error is returned. If the check passes, the error code 'DryRunOperation' is returned '.-**false** (default): Sends a normal request, returns an HTTP 2xx status code and directly creates an IPv4 Gateway.
        :param pulumi.Input[bool] enabled: Whether the IPv4 gateway is active or not. Valid values are **true** and **false**.
        :param pulumi.Input[str] ipv4_gateway_description: The description of the IPv4 gateway. The description must be 2 to 256 characters in length. It must start with a letter but cannot start with http:// or https://.
        :param pulumi.Input[str] ipv4_gateway_name: The name of the IPv4 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, periods (.), underscores (_), and hyphens (-). It must start with a letter.
        :param pulumi.Input[str] resource_group_id: The ID of the resource group to which the instance belongs.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tags of the current resource.
        :param pulumi.Input[str] vpc_id: The ID of the virtual private cloud (VPC) where you want to create the IPv4 gateway. You can create only one IPv4 gateway in a VPC.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Ipv4GatewayArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Vpc Ipv4 Gateway resource.

        For information about Vpc Ipv4 Gateway and how to use it, see [What is Ipv4 Gateway](https://www.alibabacloud.com/help/en/virtual-private-cloud/latest/createipv4gateway).

        > **NOTE:** Available in v1.181.0+.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "tf-testacc-example"
        default = alicloud.resourcemanager.ResourceGroup("default",
            display_name="tf-testAcc-rg665",
            resource_group_name=name)
        modify = alicloud.resourcemanager.ResourceGroup("modify",
            display_name="tf-testAcc-rg298",
            resource_group_name=f"{name}1")
        default_network = alicloud.vpc.Network("default",
            vpc_name=f"{name}2",
            cidr_block="10.0.0.0/8")
        default_ipv4_gateway = alicloud.vpc.Ipv4Gateway("default",
            ipv4_gateway_name=name,
            ipv4_gateway_description="tf-testAcc-Ipv4Gateway",
            resource_group_id=default.id,
            vpc_id=default_network.id)
        ```

        ## Import

        Vpc Ipv4 Gateway can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:vpc/ipv4Gateway:Ipv4Gateway example <id>
        ```

        :param str resource_name: The name of the resource.
        :param Ipv4GatewayArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(Ipv4GatewayArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 ipv4_gateway_description: Optional[pulumi.Input[str]] = None,
                 ipv4_gateway_name: Optional[pulumi.Input[str]] = None,
                 resource_group_id: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 vpc_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = Ipv4GatewayArgs.__new__(Ipv4GatewayArgs)

            __props__.__dict__["dry_run"] = dry_run
            __props__.__dict__["enabled"] = enabled
            __props__.__dict__["ipv4_gateway_description"] = ipv4_gateway_description
            __props__.__dict__["ipv4_gateway_name"] = ipv4_gateway_name
            __props__.__dict__["resource_group_id"] = resource_group_id
            __props__.__dict__["tags"] = tags
            if vpc_id is None and not opts.urn:
                raise TypeError("Missing required property 'vpc_id'")
            __props__.__dict__["vpc_id"] = vpc_id
            __props__.__dict__["create_time"] = None
            __props__.__dict__["ipv4_gateway_id"] = None
            __props__.__dict__["ipv4_gateway_route_table_id"] = None
            __props__.__dict__["status"] = None
        super(Ipv4Gateway, __self__).__init__(
            'alicloud:vpc/ipv4Gateway:Ipv4Gateway',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            create_time: Optional[pulumi.Input[str]] = None,
            dry_run: Optional[pulumi.Input[bool]] = None,
            enabled: Optional[pulumi.Input[bool]] = None,
            ipv4_gateway_description: Optional[pulumi.Input[str]] = None,
            ipv4_gateway_id: Optional[pulumi.Input[str]] = None,
            ipv4_gateway_name: Optional[pulumi.Input[str]] = None,
            ipv4_gateway_route_table_id: Optional[pulumi.Input[str]] = None,
            resource_group_id: Optional[pulumi.Input[str]] = None,
            status: Optional[pulumi.Input[str]] = None,
            tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            vpc_id: Optional[pulumi.Input[str]] = None) -> 'Ipv4Gateway':
        """
        Get an existing Ipv4Gateway resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] create_time: The creation time of the resource.
        :param pulumi.Input[bool] dry_run: Whether to PreCheck only this request. Value:-**true**: The check request is sent without creating an IPv4 Gateway. Check items include whether required parameters, request format, and business restrictions are filled in. If the check does not pass, the corresponding error is returned. If the check passes, the error code 'DryRunOperation' is returned '.-**false** (default): Sends a normal request, returns an HTTP 2xx status code and directly creates an IPv4 Gateway.
        :param pulumi.Input[bool] enabled: Whether the IPv4 gateway is active or not. Valid values are **true** and **false**.
        :param pulumi.Input[str] ipv4_gateway_description: The description of the IPv4 gateway. The description must be 2 to 256 characters in length. It must start with a letter but cannot start with http:// or https://.
        :param pulumi.Input[str] ipv4_gateway_id: Resource primary key field.
        :param pulumi.Input[str] ipv4_gateway_name: The name of the IPv4 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, periods (.), underscores (_), and hyphens (-). It must start with a letter.
        :param pulumi.Input[str] ipv4_gateway_route_table_id: ID of the route table associated with IPv4 Gateway.
        :param pulumi.Input[str] resource_group_id: The ID of the resource group to which the instance belongs.
        :param pulumi.Input[str] status: The status of the resource.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tags of the current resource.
        :param pulumi.Input[str] vpc_id: The ID of the virtual private cloud (VPC) where you want to create the IPv4 gateway. You can create only one IPv4 gateway in a VPC.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _Ipv4GatewayState.__new__(_Ipv4GatewayState)

        __props__.__dict__["create_time"] = create_time
        __props__.__dict__["dry_run"] = dry_run
        __props__.__dict__["enabled"] = enabled
        __props__.__dict__["ipv4_gateway_description"] = ipv4_gateway_description
        __props__.__dict__["ipv4_gateway_id"] = ipv4_gateway_id
        __props__.__dict__["ipv4_gateway_name"] = ipv4_gateway_name
        __props__.__dict__["ipv4_gateway_route_table_id"] = ipv4_gateway_route_table_id
        __props__.__dict__["resource_group_id"] = resource_group_id
        __props__.__dict__["status"] = status
        __props__.__dict__["tags"] = tags
        __props__.__dict__["vpc_id"] = vpc_id
        return Ipv4Gateway(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> pulumi.Output[str]:
        """
        The creation time of the resource.
        """
        return pulumi.get(self, "create_time")

    @property
    @pulumi.getter(name="dryRun")
    def dry_run(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether to PreCheck only this request. Value:-**true**: The check request is sent without creating an IPv4 Gateway. Check items include whether required parameters, request format, and business restrictions are filled in. If the check does not pass, the corresponding error is returned. If the check passes, the error code 'DryRunOperation' is returned '.-**false** (default): Sends a normal request, returns an HTTP 2xx status code and directly creates an IPv4 Gateway.
        """
        return pulumi.get(self, "dry_run")

    @property
    @pulumi.getter
    def enabled(self) -> pulumi.Output[bool]:
        """
        Whether the IPv4 gateway is active or not. Valid values are **true** and **false**.
        """
        return pulumi.get(self, "enabled")

    @property
    @pulumi.getter(name="ipv4GatewayDescription")
    def ipv4_gateway_description(self) -> pulumi.Output[Optional[str]]:
        """
        The description of the IPv4 gateway. The description must be 2 to 256 characters in length. It must start with a letter but cannot start with http:// or https://.
        """
        return pulumi.get(self, "ipv4_gateway_description")

    @property
    @pulumi.getter(name="ipv4GatewayId")
    def ipv4_gateway_id(self) -> pulumi.Output[str]:
        """
        Resource primary key field.
        """
        return pulumi.get(self, "ipv4_gateway_id")

    @property
    @pulumi.getter(name="ipv4GatewayName")
    def ipv4_gateway_name(self) -> pulumi.Output[Optional[str]]:
        """
        The name of the IPv4 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, periods (.), underscores (_), and hyphens (-). It must start with a letter.
        """
        return pulumi.get(self, "ipv4_gateway_name")

    @property
    @pulumi.getter(name="ipv4GatewayRouteTableId")
    def ipv4_gateway_route_table_id(self) -> pulumi.Output[str]:
        """
        ID of the route table associated with IPv4 Gateway.
        """
        return pulumi.get(self, "ipv4_gateway_route_table_id")

    @property
    @pulumi.getter(name="resourceGroupId")
    def resource_group_id(self) -> pulumi.Output[str]:
        """
        The ID of the resource group to which the instance belongs.
        """
        return pulumi.get(self, "resource_group_id")

    @property
    @pulumi.getter
    def status(self) -> pulumi.Output[str]:
        """
        The status of the resource.
        """
        return pulumi.get(self, "status")

    @property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        The tags of the current resource.
        """
        return pulumi.get(self, "tags")

    @property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> pulumi.Output[str]:
        """
        The ID of the virtual private cloud (VPC) where you want to create the IPv4 gateway. You can create only one IPv4 gateway in a VPC.
        """
        return pulumi.get(self, "vpc_id")

